@@ -23,17 +23,10 @@
 
 open Belenios_platform
 open Platform
-open Serializable_builtin_t
 open Serializable_t
+open Common
 
-(** Helpers for interacting with atd stuff *)
-
-type 'a reader = Yojson.Safe.lexer_state -> Lexing.lexbuf -> 'a
-type 'a writer = Buffer.t -> 'a -> unit
-
-module type GROUP = Signatures_core.GROUP
-module type MONAD = Signatures_core.MONAD
-module type RANDOM = Signatures_core.RANDOM
+include module type of Signatures_core
 
 module type ELECTION_BASE = sig
   module G : GROUP
@@ -64,7 +57,7 @@ end
 type combination_error =
   | MissingPartialDecryption
   | NotEnoughPartialDecryptions
-  | UnusedPartialDecryption
+  | InvalidPartialDecryption
 
 module type RAW_ELECTION = sig
   val raw_election : string
@@ -86,6 +79,7 @@ type ballot_id = string
 type credential_record = {
     cr_ballot : ballot_id option;
     cr_weight : weight;
+    cr_username : string option;
 }
 
 module type BBOX_OPS = sig
@@ -98,6 +92,9 @@ module type BBOX_OPS = sig
   (** Returns the credential used in previous ballot of given user, if
      any. *)
   val get_user_record : user -> string option m
+
+  (** Extracts the username from a user. *)
+  val get_username : user -> string
 end
 
 module type BBOX = sig
@@ -185,19 +182,20 @@ module type ELECTION_OPS = sig
   (** {2 Result} *)
 
   type result_type
-  type result = (result_type, elt encrypted_tally, elt partial_decryption, elt shuffle) Serializable_t.election_result
+  type result = result_type Serializable_t.election_result
   (** The election result. It contains the needed data to validate the
       result from the encrypted tally. *)
 
   val compute_result :
-    ?shuffles:elt shuffle list -> ?shufflers:shuffler list ->
-    Weight.t -> elt Serializable_t.ciphertext shape -> factor list -> elt trustees ->
+    elt encrypted_tally sized_encrypted_tally -> factor owned list -> elt trustees ->
     (result, combination_error) Stdlib.result
   (** Combine the encrypted tally and the factors from all trustees to
       produce the election result. The first argument is the number of
       tallied ballots. May raise [Invalid_argument]. *)
 
-  val check_result : elt trustees -> result -> bool
+  val check_result :
+    elt encrypted_tally sized_encrypted_tally -> factor owned list -> elt trustees ->
+    result -> bool
 end
 
 module type ELECTION = sig
