@@ -23,7 +23,6 @@ module PSerializable_j = Serializable_j
 open Belenios_platform
 open Belenios_core
 open Platform
-open Serializable_builtin_t
 open Serializable_core_j
 open Serializable_j
 open PSerializable_j
@@ -224,6 +223,12 @@ module MakeElection (W : ELECTION_DATA) (M : RANDOM) = struct
              | None -> M.return None
              | Some user -> B.get_user_record user
            in
+           let@ () = fun cont ->
+             match cr.cr_username, Option.map B.get_username user with
+             | Some x, Some y when x = y -> cont ()
+             | None, _ -> cont ()
+             | _ -> M.return (Error `WrongCredential)
+           in
            match user_record, cr.cr_ballot with
            | None, (None as x) -> M.return (Ok x)
            | None, Some _ -> M.return (Error `UsedCredential)
@@ -257,7 +262,7 @@ module MakeElection (W : ELECTION_DATA) (M : RANDOM) = struct
   end
 
   let process_ballots bs =
-    SArray (
+    `Array (
         Array.mapi (fun i q ->
             Q.process_ciphertexts q
               (List.map
@@ -291,7 +296,7 @@ module MakeElection (W : ELECTION_DATA) (M : RANDOM) = struct
            loop (i+1) (j+1)
       ) else (
         assert (j = m);
-        SArray x
+        `Array x
       )
     in
     loop 0 0
@@ -324,12 +329,12 @@ module MakeElection (W : ELECTION_DATA) (M : RANDOM) = struct
     Shape.forall (fun {alpha; beta} -> G.check alpha && G.check beta) c
 
   let rec swaps = function
-    | SAtomic x -> let* x in M.return (SAtomic x)
-    | SArray x ->
+    | `Atomic x -> let* x in M.return (`Atomic x)
+    | `Array x ->
        let rec loop i accu =
          if i >= 0
          then let* x = swaps x.(i) in loop (pred i) (x::accu)
-         else M.return (SArray (Array.of_list accu))
+         else M.return (`Array (Array.of_list accu))
        in
        loop (pred (Array.length x)) []
 
@@ -358,32 +363,35 @@ module MakeElection (W : ELECTION_DATA) (M : RANDOM) = struct
       ) c f.decryption_factors f.decryption_proofs
 
   type result_type = W.result
-  type result = (result_type, elt encrypted_tally, elt partial_decryption, elt shuffle) Serializable_t.election_result
+  type result = result_type Serializable_t.election_result
 
   module Combinator = Trustees.MakeCombinator (G)
 
-  let compute_result ?shuffles ?shufflers num_tallied encrypted_tally partial_decryptions trustees =
-    let check = check_factor encrypted_tally in
+  let compute_result encrypted_tally partial_decryptions trustees =
+    let num_tallied = encrypted_tally.sized_total_weight in
+    let et = encrypted_tally.sized_encrypted_tally in
+    let check = check_factor et in
     match Combinator.combine_factors trustees check partial_decryptions with
     | Ok factors ->
        let results =
          Shape.map2 (fun {beta; _} f ->
              beta / f
-           ) encrypted_tally factors
+           ) et factors
        in
        let raw_result =
          match results with
-         | SAtomic _ ->
+         | `Atomic _ ->
             invalid_arg "Election.compute_result: cannot compute result"
-         | SArray xs ->
+         | `Array xs ->
             Array.map2 (Q.compute_result ~num_tallied) election.e_questions xs
        in
        let result = W.cast_result raw_result in
-       Ok {num_tallied; encrypted_tally; shuffles; shufflers; partial_decryptions; result}
+       Ok {result}
     | Error e -> Error e
 
-  let check_result trustees r =
-    let {num_tallied; encrypted_tally; partial_decryptions; result; _} = r in
+  let check_result encrypted_tally partial_decryptions trustees {result} =
+    let num_tallied = encrypted_tally.sized_total_weight in
+    let encrypted_tally = encrypted_tally.sized_encrypted_tally in
     check_ciphertext encrypted_tally &&
       let check = check_factor encrypted_tally in
       match Combinator.combine_factors trustees check partial_decryptions with
@@ -395,7 +403,7 @@ module MakeElection (W : ELECTION_DATA) (M : RANDOM) = struct
              ) encrypted_tally factors
          in
          match results with
-         | SArray xs ->
+         | `Array xs ->
             Array.for_all3 (Q.check_result ~num_tallied) election.e_questions xs (result : W.result :> raw_result)
          | _ -> false
 end
