@@ -24,7 +24,6 @@ open Belenios_core
 open Platform
 open Common
 open Signatures_core
-open Serializable_builtin_t
 open Serializable_core_t
 open Question_h_t
 
@@ -94,7 +93,7 @@ module Make (M : RANDOM) (G : GROUP) = struct
       let* challenge = M.random q in
       let* response = M.random q in
       proofs.(i) <- {challenge; response};
-      commitments.(2*i) <- g **~ response *~ alpha **~ challenge;
+      commitments.(2*i) <- g **~ Z.((response + r * challenge) mod q);
       let* () = M.yield () in
       commitments.(2*i+1) <- y **~ response *~ (beta *~ d.(i)) **~ challenge;
       let* () = M.yield () in
@@ -152,7 +151,7 @@ module Make (M : RANDOM) (G : GROUP) = struct
         (* proof of m0 = 0 \/ mS = 0 (first is true) *)
         let* challenge1 = M.random q in
         let* response1 = M.random q in
-        let commitmentA1 = g **~ response1 *~ cS.alpha **~ challenge1 in
+        let commitmentA1 = g **~ Z.((response1 + rS * challenge1) mod q) in
         let* () = M.yield () in
         let commitmentB1 = y **~ response1 *~ cS.beta **~ challenge1 in
         let* w = M.random q in
@@ -176,7 +175,7 @@ module Make (M : RANDOM) (G : GROUP) = struct
         let overall_proof = Array.make (max-min+2) proof0 in
         let commitments = Array.make (2*(max-min+2)) g in
         let total_challenges = ref challenge0 in
-        commitments.(0) <- g **~ response0 *~ c0.alpha **~ challenge0;
+        commitments.(0) <- g **~ Z.((response0 + r0 * challenge0) mod q);
         let* () = M.yield () in
         commitments.(1) <- y **~ response0 *~ (c0.beta / g) **~ challenge0;
         let* () = M.yield () in
@@ -190,7 +189,7 @@ module Make (M : RANDOM) (G : GROUP) = struct
               let nbeta = cS.beta / g' in
               let j = 2*i in
               overall_proof.(i) <- {challenge; response};
-              commitments.(j) <- g **~ response *~ cS.alpha **~ challenge;
+              commitments.(j) <- g **~ Z.((response + rS * challenge) mod q);
               let* () = M.yield () in
               commitments.(j+1) <- y **~ response *~ nbeta **~ challenge;
               let* () = M.yield () in
@@ -219,7 +218,7 @@ module Make (M : RANDOM) (G : GROUP) = struct
         assert (mS = 0);
         let* challenge0 = M.random q in
         let* response0 = M.random q in
-        let commitmentA0 = g **~ response0 *~ c0.alpha **~ challenge0 in
+        let commitmentA0 = g **~ Z.((response0 + r0 * challenge0) mod q) in
         let* () = M.yield () in
         let commitmentB0 = y **~ response0 *~ c0.beta **~ challenge0 in
         let* w = M.random q in
@@ -249,7 +248,7 @@ module Make (M : RANDOM) (G : GROUP) = struct
             let nbeta = cS.beta / g' in
             let j = 2*i in
             overall_proof.(i) <- {challenge; response};
-            commitments.(j) <- g **~ response *~ cS.alpha **~ challenge;
+            commitments.(j) <- g **~ Z.((response + rS * challenge) mod q);
             let* () = M.yield () in
             commitments.(j+1) <- y **~ response *~ nbeta **~ challenge;
             let* () = M.yield () in
@@ -418,10 +417,10 @@ module Make (M : RANDOM) (G : GROUP) = struct
         | _, _ -> false
 
   let extract_ciphertexts _ a =
-    SArray (Array.map (fun x -> SAtomic x) a.choices)
+    `Array (Array.map (fun x -> `Atomic x) a.choices)
 
   let process_ciphertexts q es =
-    let neutral = SArray (Array.make (question_length q) (SAtomic dummy_ciphertext)) in
+    let neutral = `Array (Array.make (question_length q) (`Atomic dummy_ciphertext)) in
     let ( * ) = Shape.map2 eg_combine in
     let rec power b n =
       if Z.(compare n zero) > 0 then (
@@ -449,11 +448,11 @@ module Make (M : RANDOM) (G : GROUP) = struct
     fun _ x ->
     Shape.to_array x
     |> Array.map (fun i -> Weight.reduce ~total (log i))
-    |> (fun x -> RHomomorphic x)
+    |> (fun x -> `Homomorphic x)
 
   let check_result ~num_tallied _ x r =
     match r with
-    | RHomomorphic r ->
+    | `Homomorphic r ->
        Array.for_all2
          (fun x r ->
            let r = Weight.expand ~total:num_tallied r in
