@@ -22,22 +22,20 @@
 open Lwt.Syntax
 open Js_of_ocaml
 open Js_of_ocaml_tyxml
-open Belenios_core.Serializable_builtin_t
 open Belenios_core.Common
 open Belenios_api.Serializable_j
-open Belenios_tool_js_common
-open Tool_js_common
 open Tyxml_js.Html5
+open Belenios_js.Common
 open Common
 
 let context = ref `None
 
 let draft_a x =
-  let uuid = raw_string_of_uuid x.summary_uuid in
+  let uuid = Uuid.unwrap x.summary_uuid in
   a ~href:("#drafts/" ^ uuid) (if x.summary_name = "" then "(no title)" else x.summary_name)
 
 let election_a x =
-  let uuid = raw_string_of_uuid x.summary_uuid in
+  let uuid = Uuid.unwrap x.summary_uuid in
   a ~href:("#elections/" ^ uuid) (if x.summary_name = "" then "(no title)" else x.summary_name)
 
 let regexps =
@@ -81,7 +79,7 @@ let show_error main =
   Lwt.return_unit
 
 let rec show_root main =
-  let open (val !gettext) in
+  let open (val !Belenios_js.I18n.gettext) in
   main##.innerHTML := Js.string "Loading...";
   let@ () = show_in main in
   let* configuration, configuration_opt =
@@ -177,7 +175,7 @@ let rec show_root main =
       let* x = post_with_token ?ifmatch (tget ()) "drafts" |> wrap uuid_of_string in
       match x with
       | Ok uuid ->
-         Dom_html.window##.location##.hash := Js.string ("#drafts/" ^ raw_string_of_uuid uuid);
+         Dom_html.window##.location##.hash := Js.string ("#drafts/" ^ Uuid.unwrap uuid);
          Lwt.return_unit
       | Error e ->
          let@ () = show_in main in
@@ -227,8 +225,7 @@ let onload () =
     Js.Optdef.case Dom_html.window##.navigator##.language
       (fun () -> "en") Js.to_string
   in
-  let* g = I18n.get ~component:"admin" ~lang in
-  gettext := g;
+  let* () = Belenios_js.I18n.init ~dir:"" ~component:"admin" ~lang in
   let hash = parse_hash () in
   let* b =
     match hash with
