@@ -23,12 +23,10 @@ open Lwt.Syntax
 open Js_of_ocaml
 open Js_of_ocaml_tyxml
 open Belenios_core.Signatures
-open Belenios_core.Serializable_builtin_t
 open Belenios_core.Common
 open Belenios_api.Serializable_j
-open Belenios_tool_js_common
-open Tool_js_common
 open Tyxml_js.Html5
+open Belenios_js.Common
 open Common
 
 module CG = Belenios_core.Credential.MakeGenerate (LwtJsRandom)
@@ -71,7 +69,7 @@ let show main uuid =
           let container = div [] |> Tyxml_js.To_dom.of_div in
           let b =
             let@ () = button "Generate credentials" in
-            let uuid_ = uuid_of_raw_string uuid in
+            let uuid_ = Uuid.wrap uuid in
             let show_weight =
               List.exists
                 (fun v ->
