@@ -22,12 +22,11 @@
 open Lwt.Syntax
 open Js_of_ocaml
 open Js_of_ocaml_tyxml
-open Belenios_core.Serializable_builtin_t
+open Belenios_core.Signatures
 open Belenios_core.Common
 open Belenios_api.Serializable_j
-open Belenios_tool_js_common
-open Tool_js_common
 open Tyxml_js.Html5
+open Belenios_js.Common
 open Common
 
 let show_draft_main show_all uuid draft container =
@@ -76,7 +75,7 @@ let rec show_draft_voters uuid draft container =
     let i, iget = input "" in
     let b =
       let@ () = button "Import voters" in
-      let r = `Import (uuid_of_raw_string (iget ())) in
+      let r = `Import (Uuid.wrap (iget ())) in
       let* x = post_with_token ~ifmatch (string_of_voters_request r) "drafts/%s/voters" uuid in
       let@ () = show_in container in
       generic_proceed x (fun () -> show_draft_voters uuid draft container)
@@ -123,8 +122,8 @@ let rec show_draft_credentials uuid container =
      Lwt.return [b]
   | None, Some token ->
      let link = Js.to_string Dom_html.window##.location##.href ^ "@" ^ token in
-     let module X = Belenios_ui.Mails_admin.Make (I18n) in
-     let subject, body = X.mail_credential_authority !gettext link in
+     let module X = Belenios_ui.Mails_admin.Make (Belenios_js.I18n) in
+     let subject, body = X.mail_credential_authority !Belenios_js.I18n.gettext link in
      Lwt.return [
          a_mailto ~recipient:"" ~subject ~body "Send an e-mail to the credential authority";
          txt " ";
@@ -136,40 +135,56 @@ let rec show_draft_credentials uuid container =
      let t, _ = textarea (string_of_credentials x) in
      Lwt.return [t]
 
+type trustee_with_writer =
+  TWW : 'a trustee list * 'a writer -> trustee_with_writer
+
 let rec show_draft_trustees uuid container =
   let@ () = show_in container in
   let* x = get trustees_of_string "drafts/%s/trustees" uuid in
   let ifmatch = compute_ifmatch string_of_trustees x in
   let@ trustees = with_ok "trustees" x in
-  let* mode, mode_str =
-    let* x = get trustees_mode_of_string "drafts/%s/trustees-mode" uuid in
-    match x with
-    | Error e -> Lwt.return (Printf.sprintf "error (%s)" (string_of_error e), "")
-    | Ok (`Basic as x) -> Lwt.return ("basic", string_of_trustees_mode x)
-    | Ok (`Threshold threshold as x) ->
+  let mode =
+    match trustees with
+    | `Basic _ -> "basic"
+    | `Threshold t ->
        let threshold =
-         match threshold with
-         | 0 -> "not set"
-         | i -> Printf.sprintf "%d out of %d" i (List.length trustees)
+         match t.tt_threshold with
+         | None -> "not set"
+         | Some i -> Printf.sprintf "%d out of %d" i (List.length t.tt_trustees)
        in
-       Lwt.return (Printf.sprintf "threshold (%s)" threshold, string_of_trustees_mode x)
+       Printf.sprintf "threshold (%s)" threshold
   in
   let mode = div [txt "Mode:"; txt " "; txt mode] in
   let mode_set =
     let t, tget = textarea ~rows:1 ~cols:60 "" in
     let b =
-      let ifmatch = sha256_b64 mode_str in
       let@ () = button "Set mode" in
-      let* x = put_with_token ~ifmatch (tget ()) "drafts/%s/trustees-mode" uuid in
+      let@ request = fun cont ->
+        match Yojson.Safe.from_string (tget ()) with
+        | `String "Basic" -> cont `SetBasic
+        | `List [`String "Threshold"; `Int i] -> cont (`SetThreshold i)
+        | _ -> alert "Unrecognized mode"; Lwt.return_unit
+      in
+      let* x = post_with_token ?ifmatch (string_of_trustees_request request) "drafts/%s/trustees" uuid in
       let@ () = show_in container in
       generic_proceed x (fun () -> show_draft_trustees uuid container)
     in
     div [t; txt " "; b]
   in
+  let TWW (trustees, write) =
+    match trustees with
+    | `Basic x -> TWW (x.bt_trustees, write_trustee_public_key Yojson.Safe.write_json)
+    | `Threshold x -> TWW (x.tt_trustees, write_cert)
+  in
   let all_trustees =
     List.map
       (fun t ->
-        let encoded_trustee = t.trustee_address |> Js.string |> Js.encodeURIComponent |> Js.to_string in
+        let encoded_trustee =
+          Option.value ~default:"@" t.trustee_address
+          |> Js.string
+          |> Js.encodeURIComponent
+          |> Js.to_string
+        in
         let content =
           let b =
             let@ () = button "Delete" in
@@ -177,7 +192,7 @@ let rec show_draft_trustees uuid container =
             let@ () = show_in container in
             generic_proceed x (fun () -> show_draft_trustees uuid container)
           in
-          [txt (string_of_trustee t); txt " "; b]
+          [txt (string_of_trustee write t); txt " "; b]
         in
         li content
       ) trustees
@@ -186,7 +201,7 @@ let rec show_draft_trustees uuid container =
   let t2, t2get = textarea "" in
   let b =
     let@ () = button "Add trustee" in
-    let r = `Add (trustee_of_string (t2get ())) in
+    let r = `Add (trustee_of_string Yojson.Safe.read_json (t2get ())) in
     let* x = post_with_token ?ifmatch (string_of_trustees_request r) "drafts/%s/trustees" uuid in
     let@ () = show_in container in
     generic_proceed x (fun () -> show_draft_trustees uuid container)
@@ -195,7 +210,7 @@ let rec show_draft_trustees uuid container =
     let i, iget = input "" in
     let b =
       let@ () = button "Import trustees" in
-      let r = `Import (uuid_of_raw_string (iget ())) in
+      let r = `Import (Uuid.wrap (iget ())) in
       let* x = post_with_token ?ifmatch (string_of_trustees_request r) "drafts/%s/trustees" uuid in
       let@ () = show_in container in
       generic_proceed x (fun () -> show_draft_trustees uuid container)
