@@ -22,12 +22,12 @@
 open Lwt.Syntax
 open Js_of_ocaml
 open Js_of_ocaml_lwt
-open Belenios_core.Serializable_builtin_t
+open Belenios_core.Common
 open Belenios_core
 open Belenios
 open Signatures
 open Serializable_j
-open Belenios_tool_js_common.Tool_js_common
+open Belenios_js.Common
 
 let computeFingerprint = sha256_b64
 
