@@ -24,13 +24,10 @@ open Js_of_ocaml
 open Js_of_ocaml_lwt
 open Belenios_core
 open Belenios
-open Belenios_tool_js_common
-open Serializable_builtin_t
 open Serializable_j
 open Signatures
 open Common
-open Tool_js_common
-open Tool_js_i18n.Gettext
+open Belenios_js.Common
 
 module type ELECTION_LWT = ELECTION with type 'a m = 'a Lwt.t
 
@@ -66,6 +63,7 @@ type questionWidget =
   }
 
 let createHomomorphicQuestionWidget q =
+  let open (val !Belenios_js.I18n.gettext) in
   let div = Dom_html.createDiv document in
   let open Question_h_t in
   let () =
@@ -178,6 +176,7 @@ let createHomomorphicQuestionWidget q =
   {div; extract_answers}
 
 let createNonHomomorphicQuestionWidget q =
+  let open (val !Belenios_js.I18n.gettext) in
   let div = Dom_html.createDiv document in
   let open Question_nh_t in
   let () =
@@ -303,6 +302,7 @@ let arrayMapOptionGet xs =
   | Exit -> None
 
 let appendQuestionNavigation question_div sk params qs =
+  let open (val !Belenios_js.I18n.gettext) in
   let qs = Array.map createQuestionWidget qs in
   let n = Array.length qs in
   let answers = Array.make n None in
@@ -372,6 +372,7 @@ let proceedWithCredential params intro_div qs cred =
   appendQuestionNavigation e cred params qs
 
 let createStartButton params intro_div qs =
+  let open (val !Belenios_js.I18n.gettext) in
   let b = Dom_html.createButton document in
   b##.style##.fontSize := Js.string "20px";
   let t = document##createTextNode (Js.string (s_ "here")) in
@@ -409,7 +410,7 @@ let loadElection credential () =
   let params = P.election in
   set_content_with_br "election_name" params.e_name;
   set_content_with_br "election_description" params.e_description;
-  set_content "election_uuid" (raw_string_of_uuid params.e_uuid);
+  set_content "election_uuid" (Uuid.unwrap params.e_uuid);
   set_content "election_fingerprint" P.fingerprint;
   let$ e = document##getElementById (Js.string "intro") in
   match credential with
@@ -469,7 +470,7 @@ let () =
         | None -> "en"
       in
       let credential = List.assoc_opt "credential" params in
-      let* () = Tool_js_i18n.init "static" "voter" lang in
+      let* () = Belenios_js.I18n.init ~dir:"static/" ~component:"voter" ~lang in
       let () =
         let$ e = document##getElementById (Js.string "load_uuid") in
         Lwt_js_events.async (fun () ->
