@@ -23,12 +23,10 @@ open Lwt.Syntax
 open Js_of_ocaml
 open Js_of_ocaml_lwt
 open Belenios_core.Common
-open Belenios_core.Serializable_builtin_t
+open Belenios_core.Serializable_j
 open Belenios_tool_common
-open Belenios_tool_js_common
-open Tool_js_common
+open Belenios_js.Common
 open Tool_credgen
-open Tool_js_i18n.Gettext
 open Belenios_api.Serializable_j
 
 let generate uuid draft =
@@ -52,22 +50,18 @@ let generate uuid draft =
     let group = draft.draft_group
   end in
   let module X = Make (P) (LwtJsRandom) () in
-  let* privs, pubs = X.generate ids in
-  let privs =
-    List.combine ids privs
-    |> List.map (fun (id, priv) -> id ^ " " ^ priv)
-  in
-  let text_pks = (pubs |> String.concat "\n") ^ "\n" in
-  set_textarea "pks" text_pks;
-  let hash = sha256_b64 text_pks in
+  let* c = X.generate ids in
+  set_textarea "pks" (string_of_public_credentials c.public_with_ids);
+  let hash = sha256_b64 (string_of_public_credentials c.public) in
   set_content "public_creds_fp" hash;
-  let text_creds = (privs |> String.concat "\n") ^ "\n" in
+  let text_creds = (c.priv |> String.concat "\n") ^ "\n" in
   set_download "creds" "text/plain" "creds.txt" text_creds;
   set_download "voters_txt" "text/plain" "voters.txt" raw;
   set_element_display "submit_form" "inline";
   Lwt.return_unit
 
 let fill_interactivity () =
+  let open (val !Belenios_js.I18n.gettext) in
   let@ uuid, token = fun cont ->
     let hash = Dom_html.window##.location##.hash |> Js.to_string in
     match extract_uuid_and_token hash with
@@ -83,7 +77,7 @@ let fill_interactivity () =
   set_content "election_url" (build_election_url href uuid);
   let@ draft = fun cont ->
     let url = Printf.sprintf "../../api/drafts/%s" uuid in
-    let* x = get token draft_of_string url in
+    let* x = get draft_of_string url in
     match x with
     | Some x -> cont x
     | None ->
@@ -92,7 +86,7 @@ let fill_interactivity () =
   in
   let@ voters = fun cont ->
     let url = Printf.sprintf "../../api/drafts/%s/voters" uuid in
-    let* x = get token voter_list_of_string url in
+    let* x = get ~token voter_list_of_string url in
     match x with
     | Some x -> cont x
     | None ->
@@ -122,6 +116,6 @@ let fill_interactivity () =
 let () =
   Lwt.async (fun () ->
       let* _ = Lwt_js_events.onload () in
-      let* () = Tool_js_i18n.auto_init "admin" in
+      let* () = Belenios_js.I18n.auto_init "admin" in
       fill_interactivity ()
     )
