@@ -22,14 +22,11 @@
 open Lwt.Syntax
 open Js_of_ocaml
 open Js_of_ocaml_lwt
-open Belenios_core.Serializable_builtin_t
 open Belenios_core
 open Belenios
-open Belenios_tool_js_common
 open Serializable_j
 open Common
-open Tool_js_common
-open Tool_js_i18n.Gettext
+open Belenios_js.Common
 open Belenios_api.Serializable_j
 
 let election = ref None
@@ -48,6 +45,7 @@ let wrap f x =
   ); Js._false
 
 let basic_check_private_key s =
+  let open (val !Belenios_js.I18n.gettext) in
   let n = String.length s in
   let rec leading i =
     if i < n then
@@ -71,6 +69,7 @@ let basic_check_private_key s =
   in leading 0
 
 let compute_partial_decryption tally_trustee _ =
+  let open (val !Belenios_js.I18n.gettext) in
   let&|&& e = !election in
   let module P = Election.Make (struct let raw_election = e end) (LwtJsRandom) () in
   let&|&& e = !encrypted_tally in
@@ -139,7 +138,7 @@ let fill_interactivity () =
   set_form_target "pd_form" "submit-partial-decryption" uuid token;
   let@ tally_trustee = fun cont ->
     let url = Printf.sprintf "../../api/elections/%s/tally-trustee" uuid in
-    let* x = get token tally_trustee_of_string url in
+    let* x = get ~token tally_trustee_of_string url in
     match x with
     | Some x -> cont x
     | None ->
@@ -166,6 +165,6 @@ let fill_interactivity () =
 let () =
   Lwt.async (fun () ->
       let* _ = Lwt_js_events.onload () in
-      let* () = Tool_js_i18n.auto_init "admin" in
+      let* () = Belenios_js.I18n.auto_init "admin" in
       fill_interactivity ()
     )
