@@ -22,10 +22,8 @@
 open Lwt.Syntax
 open Js_of_ocaml
 open Belenios_core
-open Belenios_tool_js_common
 open Serializable_j
-open Tool_js_common
-open Tool_js_i18n.Gettext
+open Belenios_js.Common
 
 let return = Js.Opt.return
 let handler f = Dom_html.handler (fun e -> ignore (f e); Js._false)
@@ -71,6 +69,7 @@ let extractAnswer a =
   return (Js.to_string (x##.value))
 
 let extractQuestion q =
+  let open (val !Belenios_js.I18n.gettext) in
   let&& x = Dom_html.CoerceTo.input q in
   let q_question = Js.to_string (x##.value) in
   let&& p1 = q##.parentNode in
@@ -124,8 +123,8 @@ let extractQuestion q =
     )
 
 let extractTemplate () =
-  let t_name = get_input "election_name" in
-  let t_description = get_textarea "election_description" in
+  let t_name = get_input "q_election_name" in
+  let t_description = get_textarea "q_election_description" in
   let questions = document##querySelectorAll (Js.string ".question_question") in
   let t_questions =
     Array.init
@@ -141,6 +140,7 @@ let extractTemplate () =
 (* Injecting the OCaml structure into the DOM *)
 
 let rec createAnswer a =
+  let open (val !Belenios_js.I18n.gettext) in
   let container = Dom_html.createDiv document in
   container##.className := Js.string "question_answer_item";
   let t = document##createTextNode (Js.string (s_ "Answer: ")) in
@@ -176,6 +176,7 @@ let rec createAnswer a =
   container
 
 let createHomomorphicQuestionPropDiv min max blank =
+  let open (val !Belenios_js.I18n.gettext) in
   let container = Dom_html.createDiv document in
   let x = Dom_html.createDiv document in
   let t = document##createTextNode (Js.string (s_ "The voter has to choose between ")) in
@@ -220,6 +221,7 @@ let deleteQuestion q =
   return ()
 
 let rec createQuestion booth_set q =
+  let open (val !Belenios_js.I18n.gettext) in
   let question, answers, props, extra =
     match q with
     | Question.Homomorphic q ->
@@ -339,7 +341,7 @@ let rec createQuestion booth_set q =
   Dom.appendChild div_extra3 (document##createTextNode (Js.string " "));
   let more_info = Dom_html.createA document in
   Dom.appendChild div_extra3 more_info;
-  more_info##.href := Js.string "https://www.belenios.org/mixnet.html";
+  more_info##.href := Js.string Belenios_ui.Links.mixnet;
   Dom.appendChild more_info (document##createTextNode (Js.string (s_ "More information.")));
   (* selector *)
   let _type = Js.string "radio" and name = Printf.ksprintf Js.string "type%d" (gensym ()) in
@@ -418,6 +420,7 @@ let createRadioItem name checked label =
   radio, container
 
 let createBoothSelector booth_version =
+  let open (val !Belenios_js.I18n.gettext) in
   let container = Dom_html.createDiv document in
   let line1 = Dom_html.createDiv document in
   Dom.appendChild container line1;
@@ -439,6 +442,7 @@ let createBoothSelector booth_version =
   get, set, container
 
 let createTemplate (booth_get, booth_set, booth_select) template =
+  let open (val !Belenios_js.I18n.gettext) in
   let container = Dom_html.createDiv document in
   (* name *)
   let x = Dom_html.createDiv document in
@@ -446,7 +450,7 @@ let createTemplate (booth_get, booth_set, booth_select) template =
   let t = document##createTextNode (Js.string (s_ "Name of the election: ")) in
   Dom.appendChild x t;
   let h_name = Dom_html.createInput document in
-  h_name##.id := Js.string "election_name";
+  h_name##.id := Js.string "q_election_name";
   h_name##.value := Js.string template.t_name;
   Dom.appendChild x h_name;
   Dom.appendChild container x;
@@ -459,7 +463,7 @@ let createTemplate (booth_get, booth_set, booth_select) template =
   Dom.appendChild x y;
   let y = Dom_html.createDiv document in
   let h_description = Dom_html.createTextarea document in
-  h_description##.id := Js.string "election_description";
+  h_description##.id := Js.string "q_election_description";
   h_description##.value := Js.string template.t_description;
   h_description##.cols := 80;
   Dom.appendChild y h_description;
@@ -558,7 +562,7 @@ let fill_interactivity () =
 let () =
   Lwt.async (fun () ->
       let* _ = Js_of_ocaml_lwt.Lwt_js_events.onload () in
-      let* () = Tool_js_i18n.auto_init "admin" in
+      let* () = Belenios_js.I18n.auto_init "admin" in
       ignore (fill_interactivity ());
       Lwt.return_unit
     )
