@@ -22,18 +22,16 @@
 open Lwt.Syntax
 open Js_of_ocaml
 open Js_of_ocaml_lwt
-open Belenios_core.Serializable_builtin_t
 open Belenios_core
 open Belenios
-open Belenios_tool_js_common
 open Serializable_j
 open Common
-open Tool_js_common
-open Tool_js_i18n.Gettext
+open Belenios_js.Common
 
 let eta = ref 0
 
 let shuffle election ciphertexts =
+  let open (val !Belenios_js.I18n.gettext) in
   let module W = Election.Make (struct let raw_election = election end) (LwtJsRandom) () in
   let ciphertexts = nh_ciphertexts_of_string W.G.read ciphertexts in
   let full_shuffle () =
@@ -97,7 +95,7 @@ let set_nh_ciphertexts_link uuid =
 let () =
   Lwt.async (fun () ->
       let* _ = Lwt_js_events.onload () in
-      let* () = Tool_js_i18n.auto_init "admin" in
+      let* () = Belenios_js.I18n.auto_init "admin" in
       let@ uuid, token = fun cont ->
         let hash = Dom_html.window##.location##.hash |> Js.to_string in
         match extract_uuid_and_token hash with
