@@ -24,16 +24,14 @@ open Js_of_ocaml
 open Js_of_ocaml_lwt
 open Belenios_core
 open Common
-open Belenios_core.Serializable_builtin_t
 open Belenios_tool_common
-open Belenios_tool_js_common
 open Serializable_j
-open Tool_js_common
+open Belenios_js.Common
 open Tool_tkeygen
-open Tool_js_i18n.Gettext
 open Belenios_api.Serializable_j
 
 let tkeygen draft =
+  let open (val !Belenios_js.I18n.gettext) in
   let module P : PARAMS = struct
     let group = draft.draft_group
     let version = draft.draft_version
@@ -77,6 +75,7 @@ let fail msg =
   Lwt.return_unit
 
 let fill_interactivity () =
+  let open (val !Belenios_js.I18n.gettext) in
   let&&* e = document##getElementById (Js.string "interactivity") in
   let hash = Dom_html.window##.location##.hash |> Js.to_string in
   match extract_uuid_and_token hash with
@@ -87,7 +86,7 @@ let fill_interactivity () =
      set_form_target "data_form" "submit-trustee" uuid token;
      begin
        let url = Printf.sprintf "../../api/drafts/%s" uuid in
-       let* x = get token draft_of_string url in
+       let* x = get draft_of_string url in
        match x with
        | Some draft ->
           let b = Dom_html.createButton document in
@@ -106,6 +105,6 @@ let fill_interactivity () =
 let () =
   Lwt.async (fun () ->
       let* _ = Lwt_js_events.onload () in
-      let* () = Tool_js_i18n.auto_init "admin" in
+      let* () = Belenios_js.I18n.auto_init "admin" in
       fill_interactivity ()
     )
