@@ -21,18 +21,16 @@
 
 open Lwt.Syntax
 open Js_of_ocaml
-open Belenios_core.Serializable_builtin_t
 open Belenios_core
 open Common
 open Belenios
-open Belenios_tool_js_common
 open Serializable_j
 open Signatures
-open Tool_js_common
-open Tool_js_i18n.Gettext
+open Belenios_js.Common
 open Belenios_api.Serializable_j
 
 let set_step i =
+  let open (val !Belenios_js.I18n.gettext) in
   let$ e = document##getElementById (Js.string "current_step") in
   clear_content e;
   let t = Printf.sprintf (f_ "Step %d/3") i in
@@ -107,6 +105,7 @@ let fail msg =
   Lwt.return_unit
 
 let fill_interactivity () =
+  let open (val !Belenios_js.I18n.gettext) in
   let&&* e = document##getElementById (Js.string "interactivity") in
   let@ uuid, token = fun cont ->
     let hash = Dom_html.window##.location##.hash |> Js.to_string in
@@ -121,14 +120,14 @@ let fill_interactivity () =
   set_content "election_url" (build_election_url href uuid);
   let@ draft = fun cont ->
     let url = Printf.sprintf "../../api/drafts/%s" uuid in
-    let* x = get token draft_of_string url in
+    let* x = get draft_of_string url in
     match x with
     | Some x -> cont x
     | None -> fail "(token error)"
   in
   let@ pedersen = fun cont ->
     let url = Printf.sprintf "../../api/drafts/%s/trustees-pedersen" uuid in
-    let* x = get token (pedersen_of_string Yojson.Safe.read_json) url in
+    let* x = get ~token (pedersen_of_string Yojson.Safe.read_json) url in
     match x with
     | Some x -> cont x
     | None -> fail "(pedersen error)"
@@ -186,6 +185,6 @@ let fill_interactivity () =
 let () =
   Lwt.async (fun () ->
       let* _ = Js_of_ocaml_lwt.Lwt_js_events.onload () in
-      let* () = Tool_js_i18n.auto_init "admin" in
+      let* () = Belenios_js.I18n.auto_init "admin" in
       fill_interactivity ()
     )
