@@ -22,7 +22,6 @@
 open Lwt.Syntax
 open Lwt.Infix
 open Belenios_core.Common
-open Web_serializable_builtin_t
 open Web_common
 open Web_serializable_j
 
@@ -84,7 +83,7 @@ let create_account ~email user =
     | Some _ -> find_free_id (n + 1)
   in
   let* account_id = find_free_id counter in
-  let account_last_connected = now () in
+  let account_last_connected = Datetime.now () in
   let account_name =
     let x = drop_after_at user.user_name in
     if x = "" then Printf.sprintf "User #%d" account_id else x
@@ -144,10 +143,4 @@ let has_capability cap account =
   | None -> false
   | Some i -> i land (mask_of_capability cap) <> 0
 
-let check (u, a, _) = function
-  | `Id i -> List.mem a.account_id i
-  | `User u' -> u = u'
-
-let check_account a = function
-  | `Id i -> List.mem a.account_id i
-  | `User u -> List.mem u a.account_authentications
+let check a i = List.mem a.account_id i
