@@ -22,12 +22,10 @@
 open Lwt.Syntax
 open Belenios_platform.Platform
 open Belenios_core.Common
-open Belenios_core.Serializable_builtin_t
 open Belenios_core.Serializable_j
 open Belenios_core.Signatures
 open Belenios
 open Belenios_api.Serializable_j
-open Web_serializable_builtin_t
 open Web_serializable_j
 open Web_common
 open Api_generic
@@ -35,7 +33,7 @@ open Api_generic
 let with_administrator token se f =
   let@ token = Option.unwrap unauthorized token in
   match lookup_token token with
-  | Some a when Accounts.check_account a se.se_owner -> f a
+  | Some a when Accounts.check a se.se_owners -> f a
   | _ -> not_found
 
 let with_administrator_or_credential_authority token se f =
@@ -44,42 +42,25 @@ let with_administrator_or_credential_authority token se f =
     f `CredentialAuthority
   ) else (
     match lookup_token token with
-    | Some a when Accounts.check_account a se.se_owner -> f (`Administrator a)
+    | Some a when Accounts.check a se.se_owners -> f (`Administrator a)
     | _ -> not_found
   )
 
-let with_administrator_or_credential_authority_or_trustee token se f =
-  let@ token = Option.unwrap unauthorized token in
-  if token = se.se_public_creds then (
-    f `CredentialAuthority
-  ) else (
-    let@ () = fun cont ->
-      match se.se_threshold_trustees with
-      | None ->
-         begin
-           match List.find_opt (fun x -> x.st_token = token) se.se_public_keys with
-           | Some _ -> f `Trustee
-           | None -> cont ()
-         end
-      | Some trustees ->
-         begin
-           match List.find_opt (fun x -> x.stt_token = token) trustees with
-           | Some _ -> f `Trustee
-           | None -> cont ()
-         end
-    in
-    match lookup_token token with
-    | Some a when Accounts.check_account a se.se_owner -> f (`Administrator a)
-    | _ -> not_found
-  )
+let with_administrator_or_nobody token se f =
+  match token with
+  | None -> f `Nobody
+  | Some token ->
+     match lookup_token token with
+     | Some a when Accounts.check a se.se_owners -> f (`Administrator a)
+     | _ -> not_found
 
 let with_threshold_trustee token se f =
   let@ token = Option.unwrap unauthorized token in
-  match se.se_threshold_trustees with
-  | None -> not_found
-  | Some trustees ->
-     match List.find_opt (fun x -> x.stt_token = token) trustees with
-     | Some x -> f (x, trustees)
+  match se.se_trustees with
+  | `Basic _ -> not_found
+  | `Threshold t ->
+     match List.find_opt (fun x -> x.stt_token = token) t.dtp_trustees with
+     | Some x -> f (x, t)
      | None -> not_found
 
 let get_authentication se =
@@ -102,18 +83,9 @@ let api_of_draft se =
       t_administrator = Some (Option.value se.se_administrator ~default:"");
     }
   in
-  let* draft_owners =
-    match se.se_owner with
-    | `Id i -> Lwt.return i
-    | `User u ->
-       let* x = Accounts.get_account u in
-       match x with
-       | None -> Lwt.return []
-       | Some a -> Lwt.return [a.account_id]
-  in
   Lwt.return {
-    draft_version = Option.value se.se_version ~default:0;
-    draft_owners;
+    draft_version = se.se_version;
+    draft_owners = se.se_owners;
     draft_questions;
     draft_languages = Option.value se.se_metadata.e_languages ~default:[];
     draft_contact = se.se_metadata.e_contact;
@@ -126,7 +98,7 @@ let assert_ msg b f =
   if b then f () else raise (Error msg)
 
 let draft_of_api a se d =
-  let version = Option.value se.se_version ~default:0 in
+  let version = se.se_version in
   let () =
     if d.draft_version <> version then
       raise (Error "cannot change version")
@@ -181,20 +153,20 @@ let draft_of_api a se d =
   {
     se with
     se_metadata;
-    se_owner = `Id d.draft_owners;
+    se_owners = d.draft_owners;
     se_questions = d.draft_questions;
     se_administrator = d.draft_questions.t_administrator;
     se_group;
   }
 
 let delete_draft uuid =
-  let* () = rmdir !!(raw_string_of_uuid uuid) in
+  let* () = rmdir !!(Uuid.unwrap uuid) in
   Web_persist.clear_elections_by_owner_cache ()
 
 let generate_uuid () =
   let length = !Web_config.uuid_length in
   let* token = generate_token ?length () in
-  Lwt.return (uuid_of_raw_string token)
+  Lwt.return (Uuid.wrap token)
 
 let post_drafts account draft =
   let@ () = fun cont ->
@@ -203,12 +175,12 @@ let post_drafts account draft =
     else
       Lwt.(cont () >>= return_some)
   in
-  let owner = `Id [account.account_id] in
+  let owners = [account.account_id] in
   let* uuid = generate_uuid () in
   let* token = generate_token () in
   let se_metadata =
     {
-      e_owner = Some owner;
+      e_owners = owners;
       e_auth_config = None;
       e_cred_authority = None;
       e_trustees = None;
@@ -228,25 +200,21 @@ let post_drafts account draft =
   in
   let se =
     {
-      se_version = Some (List.hd supported_crypto_versions);
-      se_owner = owner;
+      se_version = List.hd supported_crypto_versions;
+      se_owners = owners;
       se_group = !Web_config.default_group;
       se_voters = [];
       se_questions;
-      se_public_keys = [];
+      se_trustees = `Basic {dbp_trustees = []};
       se_metadata;
       se_public_creds = token;
       se_public_creds_received = false;
-      se_threshold = None;
-      se_threshold_trustees = None;
-      se_threshold_parameters = None;
-      se_threshold_error = None;
-      se_creation_date = Some (now ());
+      se_creation_date = Some (Datetime.now ());
       se_administrator = None;
     }
   in
   let se = draft_of_api account se draft in
-  let* () = Lwt_unix.mkdir !!(raw_string_of_uuid uuid) 0o700 in
+  let* () = Lwt_unix.mkdir !!(Uuid.unwrap uuid) 0o700 in
   let* () = Web_persist.set_draft_election uuid se in
   let* () = Web_persist.clear_elections_by_owner_cache () in
   Lwt.return uuid
@@ -358,7 +326,11 @@ let get_draft_credentials who uuid se =
     else
       Some se.se_public_creds
   in
-  let* credentials_public = read_file ~uuid "public_creds.txt" in
+  let* credentials_public =
+    let* x = read_file_single_line ~uuid "public_creds.json" in
+    let&* x in
+    Lwt.return_some (public_credentials_of_string x)
+  in
   let* credentials_private =
     match who with
     | `Administrator _ ->
@@ -396,7 +368,7 @@ let generate_credentials_on_server send uuid se =
           weight <> None
         ) se.se_voters
     in
-    let version = Option.value se.se_version ~default:0 in
+    let version = se.se_version in
     let module G = (val Group.of_string ~version se.se_group : GROUP) in
     let module CMap = Map.Make (G) in
     let module CD = Belenios_core.Credential.MakeDerive (G) in
@@ -424,7 +396,8 @@ let generate_credentials_on_server send uuid se =
              else cred
            )
     in
-    let* () = write_file ~uuid "public_creds.txt" public_creds in
+    let public_creds = string_of_public_credentials public_creds in
+    let* () = write_file ~uuid "public_creds.json" [public_creds] in
     se.se_public_creds_received <- true;
     let* () = Web_persist.set_draft_election uuid se in
     Lwt.return (Ok jobs)
@@ -438,59 +411,113 @@ let exn_of_generate_credentials_on_server_error = function
 
 let submit_public_credentials uuid se credentials =
   let () = if se.se_voters = [] then raise (Error "No voters") in
-  let version = Option.value se.se_version ~default:0 in
+  let version = se.se_version in
   let module G = (val Group.of_string ~version se.se_group : GROUP) in
-  let weights =
-    List.fold_left
-      (fun (i, weights) x ->
-        try
-          let x, w = extract_weight x in
-          let x = G.of_string x in
-          if not (G.check x) then raise Exit;
-          i + 1, w :: weights
-        with _ ->
-          raise (Error (Printf.sprintf "invalid credential at index %d" i))
-      ) (0, []) credentials
-    |> (fun (_, weights) -> List.sort Weight.compare weights)
-  in
-  let expected_weights =
+  let usernames =
     List.fold_left
       (fun accu {sv_id; _} ->
-        let _, _, weight = split_identity sv_id in
-        weight :: accu
-      ) [] se.se_voters
-    |> List.sort Weight.compare
+        let _, username, weight = split_identity sv_id in
+        if SMap.mem username accu then (
+          raise (Error (Printf.sprintf "duplicate username %s" username))
+        ) else (
+          SMap.add username (weight, ref false) accu
+        )
+      ) SMap.empty se.se_voters
   in
-  if weights <> expected_weights then raise (Error "discrepancy in weights");
-  let* () = write_file ~uuid "public_creds.txt" credentials in
+  let _ =
+    List.fold_left
+      (fun (i, accu) x ->
+        let invalid fmt =
+          Printf.ksprintf
+            (fun x ->
+              raise (Error (Printf.sprintf "invalid %s at index %d" x i))
+            ) fmt
+        in
+        let cred, weight, username =
+          match String.split_on_char ',' x with
+          | [c; ""; u] -> G.of_string c, Weight.one, u
+          | [c; w; u] -> G.of_string c, Weight.of_string w, u
+          | _ -> invalid "record"
+        in
+        let cred_s = G.to_string cred in
+        let () =
+          match SMap.find_opt username usernames with
+          | None -> invalid "username %s" username
+          | Some (w, used) ->
+             if !used then
+               invalid "duplicate username %s" username
+             else if Weight.compare w weight <> 0 then
+               invalid "differing weight"
+             else if SSet.mem cred_s accu then
+               invalid "duplicate credential"
+             else if not (G.check cred) then
+               invalid "public credential"
+             else used := true
+        in
+        i + 1, SSet.add cred_s accu
+      ) (0, SSet.empty) credentials
+  in
+  let credentials = string_of_public_credentials credentials in
+  let* () = write_file ~uuid "public_creds.json" [credentials] in
   se.se_public_creds_received <- true;
   Web_persist.set_draft_election uuid se
 
-let get_draft_trustees se =
-  match se.se_threshold_trustees with
-  | None ->
-     List.filter_map
-       (fun t ->
-         if t.st_id = "server" then
-           None
-         else
-           Some {
-               trustee_address = t.st_id;
-               trustee_name = Option.value t.st_name ~default:"";
-               trustee_token = Some t.st_token;
-               trustee_state = Some (if t.st_public_key = "" then 0 else 1);
-             }
-       ) se.se_public_keys
-  | Some ts ->
-     List.map
-       (fun t ->
-         {
-           trustee_address = t.stt_id;
-           trustee_name = Option.value t.stt_name ~default:"";
-           trustee_token = Some t.stt_token;
-           trustee_state = Some (Option.value t.stt_step ~default:0);
-         }
-       ) ts
+let get_draft_trustees ~is_admin se =
+  match se.se_trustees with
+  | `Basic x ->
+     let bt_trustees =
+       List.filter_map
+         (fun t ->
+           if t.st_id = "server" then (
+             None
+           ) else (
+             let trustee_state, trustee_key =
+               if t.st_public_key = "" then
+                 Some 0, None
+               else
+                 Some 1, Some (trustee_public_key_of_string Yojson.Safe.read_json t.st_public_key)
+             in
+             let trustee_address, trustee_token, trustee_state =
+               if is_admin then
+                 Some t.st_id, Some t.st_token, trustee_state
+               else
+                 None, None, None
+             in
+             Some {
+                 trustee_address;
+                 trustee_name = Option.value t.st_name ~default:"";
+                 trustee_token;
+                 trustee_state;
+                 trustee_key;
+               }
+           )
+         ) x.dbp_trustees
+     in
+     `Basic {bt_trustees}
+  | `Threshold x ->
+     let tt_trustees =
+       List.map
+         (fun t ->
+           let trustee_address, trustee_token, trustee_state =
+             if is_admin then
+               Some t.stt_id, Some t.stt_token, Some (Option.value t.stt_step ~default:0)
+             else
+               None, None, None
+           in
+           {
+             trustee_address;
+             trustee_name = Option.value t.stt_name ~default:"";
+             trustee_token;
+             trustee_state;
+             trustee_key = t.stt_cert;
+           }
+         ) x.dtp_trustees
+     in
+     `Threshold
+       {
+         tt_threshold = x.dtp_threshold;
+         tt_trustees;
+       }
 
 let check_address address =
   if not @@ is_email address then
@@ -502,9 +529,9 @@ let ensure_none label x =
 
 let generate_server_trustee se =
   let st_id = "server" and st_token = "" in
-  let version = Option.value se.se_version ~default:0 in
+  let version = se.se_version in
   let module G = (val Group.of_string ~version se.se_group) in
-  let module Trustees = (val Trustees.get_by_version (Option.value se.se_version ~default:0)) in
+  let module Trustees = (val Trustees.get_by_version version) in
   let module K = Trustees.MakeSimple (G) (LwtRandom) in
   let* private_key = K.generate () in
   let* public_key = K.prove private_key in
@@ -514,13 +541,18 @@ let generate_server_trustee se =
   Lwt.return {st_id; st_token; st_public_key; st_private_key; st_name}
 
 let post_draft_trustees uuid se t =
-  let () = check_address t.trustee_address in
+  let address =
+    match t.trustee_address with
+    | Some x -> check_address x; x
+    | None -> raise (Error "missing address")
+  in
   let () = ensure_none "token" t.trustee_token in
   let () = ensure_none "state" t.trustee_state in
-  match se.se_threshold_trustees with
-  | None ->
+  let () = ensure_none "key" t.trustee_key in
+  match se.se_trustees with
+  | `Basic x ->
      let* ts =
-       let ts = se.se_public_keys in
+       let ts = x.dbp_trustees in
        if List.exists (fun x -> x.st_id = "server") ts then
          Lwt.return ts
        else
@@ -528,39 +560,38 @@ let post_draft_trustees uuid se t =
          Lwt.return (ts @ [server])
      in
      let () =
-       if List.exists (fun x -> x.st_id = t.trustee_address) ts then
+       if List.exists (fun x -> x.st_id = address) ts then
          raise (Error "address already used")
      in
      let* st_token = generate_token () in
      let t =
        {
-         st_id = t.trustee_address;
+         st_id = address;
          st_name = Some t.trustee_name;
          st_public_key = "";
          st_private_key = None;
          st_token;
        }
      in
-     let se_public_keys = ts @ [t] in
-     let se = {se with se_public_keys} in
+     x.dbp_trustees <- ts @ [t];
      Web_persist.set_draft_election uuid se
-  | Some ts ->
+  | `Threshold x ->
+     let ts = x.dtp_trustees in
      let () =
-       if List.exists (fun x -> x.stt_id = t.trustee_address) ts then
+       if List.exists (fun x -> x.stt_id = address) ts then
          raise (Error "address already used")
      in
      let* stt_token = generate_token () in
      let t =
        {
-         stt_id = t.trustee_address;
+         stt_id = address;
          stt_name = Some t.trustee_name;
          stt_token; stt_step = None;
          stt_cert = None; stt_polynomial = None;
          stt_vinput = None; stt_voutput = None;
        }
      in
-     let se_threshold_trustees = Some (ts @ [t]) in
-     let se = {se with se_threshold_trustees} in
+     x.dtp_trustees <- ts @ [t];
      Web_persist.set_draft_election uuid se
 
 let rec filter_out_first f = function
@@ -574,34 +605,42 @@ let rec filter_out_first f = function
      )
 
 let delete_draft_trustee uuid se trustee =
-  match se.se_threshold_trustees with
-  | None ->
-     let touched, ts = filter_out_first (fun x -> x.st_id = trustee) se.se_public_keys in
+  match se.se_trustees with
+  | `Basic x ->
+     let ts = x.dbp_trustees in
+     let touched, ts = filter_out_first (fun x -> x.st_id = trustee) ts in
      if touched then (
-       let* () = Web_persist.set_draft_election uuid {se with se_public_keys = ts} in
+       x.dbp_trustees <- ts;
+       let* () = Web_persist.set_draft_election uuid se in
        Lwt.return_true
      ) else (
        Lwt.return_false
      )
-  | Some ts ->
+  | `Threshold x ->
+     let ts = x.dtp_trustees in
      let touched, ts = filter_out_first (fun x -> x.stt_id = trustee) ts in
      if touched then (
-       let* () = Web_persist.set_draft_election uuid {se with se_threshold_trustees = Some ts} in
+       x.dtp_trustees <- ts;
+       let* () = Web_persist.set_draft_election uuid se in
        Lwt.return_true
      ) else (
        Lwt.return_false
      )
 
 let set_threshold uuid se threshold =
-  match se.se_threshold_trustees with
-  | None | Some [] -> Lwt.return @@ Stdlib.Error `NoTrustees
-  | Some ts ->
+  match se.se_trustees with
+  | `Basic _ ->
+     Lwt.return @@ Stdlib.Error `NoTrustees
+  | `Threshold x when x.dtp_trustees = [] ->
+     Lwt.return @@ Stdlib.Error `NoTrustees
+  | `Threshold x ->
+     let ts = x.dtp_trustees in
      let maybe_threshold, step =
        if threshold = 0 then None, None else Some threshold, Some 1
      in
      if 0 <= threshold && threshold < List.length ts then (
        List.iter (fun t -> t.stt_step <- step) ts;
-       se.se_threshold <- maybe_threshold;
+       x.dtp_threshold <- maybe_threshold;
        let* () = Web_persist.set_draft_election uuid se in
        Lwt.return @@ Ok ()
      ) else (
@@ -609,25 +648,26 @@ let set_threshold uuid se threshold =
      )
 
 let get_draft_trustees_mode se =
-  match se.se_threshold_trustees with
-  | None -> `Basic
-  | Some _ -> `Threshold (Option.value se.se_threshold ~default:0)
+  match se.se_trustees with
+  | `Basic _ -> `Basic
+  | `Threshold x -> `Threshold (Option.value x.dtp_threshold ~default:0)
 
 let put_draft_trustees_mode uuid se mode =
   match get_draft_trustees_mode se, mode with
   | a, b when a = b -> Lwt.return_unit
   | _, `Basic ->
-     let se = {se with se_public_keys = []; se_threshold_trustees = None} in
+     se.se_trustees <- `Basic {dbp_trustees = []};
      Web_persist.set_draft_election uuid se
   | `Basic, `Threshold 0 ->
-     let se =
+     let dtp =
        {
-         se with
-         se_public_keys = [];
-         se_threshold_trustees = Some [];
-         se_threshold = None;
+         dtp_threshold = None;
+         dtp_trustees = [];
+         dtp_parameters = None;
+         dtp_error = None;
        }
      in
+     se.se_trustees <- `Threshold dtp;
      Web_persist.set_draft_election uuid se
   | `Threshold _, `Threshold threshold ->
      begin
@@ -659,11 +699,11 @@ let get_draft_status uuid se =
       status_private_credentials_downloaded;
       status_trustees_ready =
         begin
-          match se.se_threshold_trustees with
-          | None ->
-             List.for_all (fun t -> t.st_public_key <> "") se.se_public_keys
-          | Some ts ->
-             List.for_all (fun t -> t.stt_step = Some 7) ts
+          match se.se_trustees with
+          | `Basic x ->
+             List.for_all (fun t -> t.st_public_key <> "") x.dbp_trustees
+          | `Threshold x ->
+             List.for_all (fun t -> t.stt_step = Some 7) x.dtp_trustees
         end;
       status_nh_and_weights_compatible =
         begin
@@ -692,10 +732,10 @@ let dump_passwords uuid db =
   List.map (fun line -> String.concat "," line) db |>
     write_file ~uuid "passwords.csv"
 
-let validate_election account uuid se =
+let validate_election uuid se =
   let* s = get_draft_status uuid se in
-  let version = Option.value se.se_version ~default:0 in
-  let uuid_s = raw_string_of_uuid uuid in
+  let version = se.se_version in
+  let uuid_s = Uuid.unwrap uuid in
   (* convenience tests *)
   let () =
     if se.se_questions.t_name = "" then
@@ -735,10 +775,11 @@ let validate_election account uuid se =
   let module K = Trustees.MakeCombinator (G) in
   let module KG = Trustees.MakeSimple (G) (LwtRandom) in
   let* trustee_names, trustees, private_keys =
-    match se.se_threshold_trustees with
-    | None ->
+    match se.se_trustees with
+    | `Basic x ->
+       let ts = x.dbp_trustees in
        let* trustee_names, trustees, private_key =
-         match se.se_public_keys with
+         match ts with
          | [] ->
             let* private_key = KG.generate () in
             let* public_key = KG.prove private_key in
@@ -750,7 +791,7 @@ let validate_election account uuid se =
                   match st_private_key with
                   | Some x -> x :: accu
                   | None -> accu
-                ) [] se.se_public_keys
+                ) [] ts
             in
             let private_key = match private_key with
               | [x] -> `KEY x
@@ -758,19 +799,20 @@ let validate_election account uuid se =
             in
             Lwt.return
               begin
-                (List.map (fun {st_id; _} -> st_id) se.se_public_keys),
+                (List.map (fun {st_id; _} -> st_id) ts),
                 (List.map
                    (fun {st_public_key; st_name; _} ->
                      let pk = trustee_public_key_of_string G.read st_public_key in
                      let pk = { pk with trustee_name = st_name } in
                      `Single pk
-                   ) se.se_public_keys),
+                   ) ts),
                 private_key
               end
        in
        Lwt.return (trustee_names, trustees, private_key)
-    | Some ts ->
-       match se.se_threshold_parameters with
+    | `Threshold x ->
+       let ts = x.dtp_trustees in
+       match x.dtp_parameters with
        | None -> raise (Error "key establishment not finished")
        | Some tp ->
           let tp = threshold_parameters_of_string G.read tp in
@@ -803,22 +845,17 @@ let validate_election account uuid se =
   in
   let y = K.combine_keys trustees in
   (* election parameters *)
-  let e_owner =
-    match se.se_owner with
-    | `Id _ as x -> Some x
-    | `User _ -> Some (`Id [account.account_id])
-  in
   let metadata =
     {
       se.se_metadata with
       e_trustees = Some trustee_names;
-      e_owner;
+      e_owners = se.se_owners;
     }
   in
   let template = se.se_questions in
   let params =
     {
-      e_version = Option.value se.se_version ~default:0;
+      e_version = se.se_version;
       e_description = template.t_description;
       e_name = template.t_name;
       e_questions = template.t_questions;
@@ -844,18 +881,42 @@ let validate_election account uuid se =
             Lwt_io.write oc "\n") xs)
   in
   let open Belenios_core.Serializable_j in
-  let* () = create_file "trustees.json" (string_of_trustees G.write) [trustees] in
   let* () = create_file "voters.txt" (fun x -> x.sv_id) se.se_voters in
   let* () = create_file "metadata.json" string_of_metadata [metadata] in
-  let* () = create_file "election.json" (fun x -> x) [raw_election] in
-  let* () = create_file "ballots.jsons" (fun x -> x) [] in
   (* initialize credentials *)
-  let* () =
-    let fname = uuid /// "public_creds.txt" in
-    let* file = read_file fname in
+  let* public_creds =
+    let fname = uuid /// "public_creds.json" in
+    let* file = read_file_single_line fname in
     match file with
-    | Some xs -> Web_persist.init_credential_mapping uuid xs
-    | None -> Lwt.return_unit
+    | Some x ->
+       let x = public_credentials_of_string x |> List.map strip_cred in
+       let* () = Web_persist.init_credential_mapping uuid x in
+       Lwt.return x
+    | None -> Lwt.fail @@ Failure "no public credentials"
+  in
+  (* initialize events *)
+  let* () =
+    let raw_trustees = string_of_trustees G.write trustees in
+    let raw_public_creds = string_of_public_credentials public_creds in
+    let setup_election = Hash.hash_string raw_election in
+    let setup_trustees = Hash.hash_string raw_trustees in
+    let setup_credentials = Hash.hash_string raw_public_creds in
+    let setup_data =
+      {
+        setup_election;
+        setup_trustees;
+        setup_credentials;
+      }
+    in
+    let setup_data_s = string_of_setup_data setup_data in
+    Web_events.append ~lock:false ~uuid
+      [
+        Data raw_election;
+        Data raw_trustees;
+        Data raw_public_creds;
+        Data setup_data_s;
+        Event (`Setup, Some (Hash.hash_string setup_data_s));
+      ]
   in
   (* create file with private keys, if any *)
   let* () =
@@ -887,7 +948,7 @@ let validate_election account uuid se =
   (* finish *)
   let* () = Web_persist.set_election_state uuid `Open in
   let* dates = Web_persist.get_election_dates uuid in
-  Web_persist.set_election_dates uuid {dates with e_finalization = Some (now ())}
+  Web_persist.set_election_dates uuid {dates with e_finalization = Some (Datetime.now ())}
 
 let merge_voters a b f =
   let weights =
@@ -946,9 +1007,9 @@ let import_trustees uuid se from metadata =
   | None -> Lwt.return @@ Stdlib.Error `None
   | Some names ->
      let* trustees = Web_persist.get_trustees from in
-     let version = Option.value se.se_version ~default:0 in
+     let version = se.se_version in
      let module G = (val Group.of_string ~version se.se_group : GROUP) in
-     let module Trustees = (val Trustees.get_by_version (Option.value se.se_version ~default:0)) in
+     let module Trustees = (val Trustees.get_by_version version) in
      let module K = Trustees.MakeCombinator (G) in
      let trustees = trustees_of_string G.read trustees in
      if not (K.check trustees) then
@@ -980,9 +1041,15 @@ let import_trustees uuid se from metadata =
          in
          match x with
          | Ok se_threshold_trustees ->
-            se.se_threshold <- Some t.t_threshold;
-            se.se_threshold_trustees <- Some se_threshold_trustees;
-            se.se_threshold_parameters <- Some (string_of_threshold_parameters G.write t);
+            let dtp =
+              {
+                dtp_threshold = Some t.t_threshold;
+                dtp_trustees = se_threshold_trustees;
+                dtp_parameters = Some (string_of_threshold_parameters G.write t);
+                dtp_error = None;
+              }
+            in
+            se.se_trustees <- `Threshold dtp;
             let* () = Web_persist.set_draft_election uuid se in
             Lwt.return @@ Ok `Threshold
          | Stdlib.Error _ as x -> Lwt.return x
@@ -1021,29 +1088,30 @@ let import_trustees uuid se from metadata =
                    let st_name = public_key.trustee_name in
                    Lwt.return {st_id; st_token; st_public_key; st_private_key; st_name})
           in
-          se.se_public_keys <- ts;
+          se.se_trustees <- `Basic {dbp_trustees = ts};
           let* () = Web_persist.set_draft_election uuid se in
           Lwt.return @@ Ok `Basic
 
 let check_owner account uuid cont =
   let* metadata = Web_persist.get_election_metadata uuid in
-  match metadata.e_owner with
-  | Some o when Accounts.check_account account o -> cont metadata
-  | _ -> unauthorized
+  if Accounts.check account metadata.e_owners then
+    cont metadata
+  else
+    unauthorized
 
-let post_draft_status account uuid se = function
+let post_draft_status uuid se = function
   | `SetDownloaded ->
      let* () = set_downloaded uuid in
      ok
   | `ValidateElection ->
-     let* () = validate_election account uuid se in
+     let* () = validate_election uuid se in
      ok
 
 let dispatch_draft ~token ~ifmatch endpoint method_ body uuid se =
   match endpoint with
   | [] ->
      begin
-       let@ who = with_administrator_or_credential_authority_or_trustee token se in
+       let@ who = with_administrator_or_nobody token se in
        let get () =
          let* x = api_of_draft se in
          Lwt.return @@ string_of_draft x
@@ -1056,7 +1124,7 @@ let dispatch_draft ~token ~ifmatch endpoint method_ body uuid se =
           let@ () = handle_generic_error in
           let update_cache =
             draft.draft_questions.t_name <> se.se_questions.t_name
-            || se.se_owner <> `Id draft.draft_owners
+            || se.se_owners <> draft.draft_owners
           in
           let se = draft_of_api account se draft in
           let* () = Web_persist.set_draft_election uuid se in
@@ -1067,11 +1135,11 @@ let dispatch_draft ~token ~ifmatch endpoint method_ body uuid se =
               Lwt.return_unit
           in
           ok
-       | `POST, `Administrator account ->
+       | `POST, `Administrator _ ->
           let@ () = handle_ifmatch ifmatch get in
           let@ x = body.run draft_request_of_string in
           let@ () = handle_generic_error in
-          post_draft_status account uuid se x
+          post_draft_status uuid se x
        | `DELETE, `Administrator _ ->
           let@ () = handle_ifmatch ifmatch get in
           let@ () = handle_generic_error in
@@ -1182,38 +1250,21 @@ let dispatch_draft ~token ~ifmatch endpoint method_ body uuid se =
           )
        | _ -> method_not_allowed
      end
-  | ["trustees-mode"] ->
-     begin
-       let@ _ = with_administrator token se in
-       let get () =
-         let x = get_draft_trustees_mode se in
-         Lwt.return @@ string_of_trustees_mode x
-       in
-       match method_ with
-       | `GET -> handle_get get
-       | `PUT ->
-          let@ () = handle_ifmatch ifmatch get in
-          let@ mode = body.run trustees_mode_of_string in
-          let@ () = handle_generic_error in
-          let* () = put_draft_trustees_mode uuid se mode in
-          ok
-       | _ -> method_not_allowed
-     end
   | ["trustees-pedersen"] ->
      begin
-       let@ trustee, trustees = with_threshold_trustee token se in
+       let@ trustee, dtp = with_threshold_trustee token se in
        let get () =
          let pedersen_certs =
            List.fold_left (fun accu x ->
                match x.stt_cert with
                | None -> accu
                | Some c -> c :: accu
-             ) [] trustees
+             ) [] dtp.dtp_trustees
            |> List.rev |> Array.of_list
          in
          let r =
            {
-             pedersen_threshold = Option.value ~default:0 se.se_threshold;
+             pedersen_threshold = Option.value ~default:0 dtp.dtp_threshold;
              pedersen_step = Option.value ~default:0 trustee.stt_step;
              pedersen_certs;
              pedersen_vinput = trustee.stt_vinput;
@@ -1228,15 +1279,16 @@ let dispatch_draft ~token ~ifmatch endpoint method_ body uuid se =
      end
   | ["trustees"] ->
      begin
-       let@ account = with_administrator token se in
-       let get () =
-         let x = get_draft_trustees se in
+       let@ who = with_administrator_or_nobody token se in
+       let get is_admin () =
+         let x = get_draft_trustees ~is_admin se in
          Lwt.return @@ string_of_trustees x
        in
-       match method_ with
-       | `GET -> handle_get get
-       | `POST ->
-          let@ () = handle_ifmatch ifmatch get in
+       match method_, who with
+       | `GET, `Nobody -> handle_get (get false)
+       | `GET, `Administrator _ -> handle_get (get true)
+       | `POST, `Administrator account ->
+          let@ () = handle_ifmatch ifmatch (get true) in
           let@ request = body.run trustees_request_of_string in
           let@ () = handle_generic_error in
           begin
@@ -1244,6 +1296,12 @@ let dispatch_draft ~token ~ifmatch endpoint method_ body uuid se =
             | `Add trustee ->
                let* () = post_draft_trustees uuid se trustee in
                ok
+            | `SetBasic ->
+               let* () = put_draft_trustees_mode uuid se `Basic in
+               ok
+            | `SetThreshold t ->
+               let* () = put_draft_trustees_mode uuid se (`Threshold t) in
+               ok
             | `Import from ->
                let@ metadata = check_owner account from in
                let* x = import_trustees uuid se from metadata in
@@ -1295,7 +1353,7 @@ let dispatch ~token ~ifmatch endpoint method_ body =
          let elections =
            List.fold_left
              (fun accu (kind, summary_uuid, date, summary_name) ->
-               let summary_date = unixfloat_of_datetime date in
+               let summary_date = Datetime.to_unixfloat date in
                let summary_kind = None in
                if kind = `Draft then
                  {summary_uuid; summary_name; summary_date; summary_kind} :: accu
@@ -1320,7 +1378,7 @@ let dispatch ~token ~ifmatch endpoint method_ body =
        | _ -> method_not_allowed
      end
   | uuid :: endpoint ->
-     let@ uuid = Option.unwrap bad_request (Option.wrap uuid_of_raw_string uuid) in
+     let@ uuid = Option.unwrap bad_request (Option.wrap Uuid.wrap uuid) in
      let* se = Web_persist.get_draft_election uuid in
      let@ se = Option.unwrap not_found se in
      dispatch_draft ~token ~ifmatch endpoint method_ body uuid se
