@@ -21,9 +21,7 @@
 
 open Lwt.Syntax
 open Belenios_core.Common
-open Belenios_core.Serializable_builtin_t
 open Belenios_core.Serializable_j
-open Web_serializable_builtin_t
 open Web_serializable_j
 open Belenios_api.Serializable_j
 open Web_common
@@ -31,8 +29,8 @@ open Api_generic
 
 let with_administrator token metadata f =
   let@ token = Option.unwrap unauthorized token in
-  match lookup_token token, metadata.e_owner with
-  | Some a, Some o when Accounts.check_account a o -> f a
+  match lookup_token token with
+  | Some a when Accounts.check a metadata.e_owners -> f a
   | _ -> unauthorized
 
 let find_trustee_id uuid token =
@@ -71,46 +69,41 @@ let with_tally_trustee token uuid f =
   | None -> unauthorized
 
 let get_election_status uuid =
-  let* s = Web_persist.get_election_state uuid in
+  let* status_state = Web_persist.get_election_state uuid in
   let* d = Web_persist.get_election_dates uuid in
-  let status_state =
-    match s with
-    | `EncryptedTally _ -> `EncryptedTally
-    | (`Open | `Closed | `Shuffling | `Tallied | `Archived) as x -> x
-  in
   let status_auto_archive_date =
     match status_state with
     | `Tallied ->
        let t = Option.value d.e_tally ~default:default_tally_date in
-       Some (unixfloat_of_datetime @@ datetime_add t (day days_to_archive))
+       Some (Datetime.to_unixfloat @@ Period.add t (Period.day days_to_archive))
     | _ -> None
   in
   let status_auto_delete_date =
     match status_state with
     | `Open | `Closed | `Shuffling | `EncryptedTally ->
        let t = Option.value d.e_finalization ~default:default_validation_date in
-       unixfloat_of_datetime @@ datetime_add t (day days_to_delete)
+       Datetime.to_unixfloat @@ Period.add t (Period.day days_to_delete)
     | `Tallied ->
        let t = Option.value d.e_tally ~default:default_tally_date in
-       unixfloat_of_datetime @@ datetime_add t (day (days_to_archive + days_to_delete))
+       Datetime.to_unixfloat @@ Period.add t (Period.day (days_to_archive + days_to_delete))
     | `Archived ->
        let t = Option.value d.e_archive ~default:default_archive_date in
-       unixfloat_of_datetime @@ datetime_add t (day days_to_delete)
+       Datetime.to_unixfloat @@ Period.add t (Period.day days_to_delete)
   in
   let* postpone = Web_persist.get_election_result_hidden uuid in
   Lwt.return {
       status_state;
       status_auto_archive_date;
       status_auto_delete_date;
-      status_postpone_date = Option.map unixfloat_of_datetime postpone;
+      status_postpone_date = Option.map Datetime.to_unixfloat postpone;
     }
 
 let get_election_automatic_dates uuid =
   let* d = Web_persist.get_election_dates uuid in
   Lwt.return
     {
-      auto_date_open = Option.map unixfloat_of_datetime d.e_auto_open;
-      auto_date_close = Option.map unixfloat_of_datetime d.e_auto_close;
+      auto_date_open = Option.map Datetime.to_unixfloat d.e_auto_open;
+      auto_date_close = Option.map Datetime.to_unixfloat d.e_auto_close;
     }
 
 let set_election_state uuid state =
@@ -139,37 +132,16 @@ let open_election uuid = set_election_state uuid `Open
 let close_election uuid = set_election_state uuid `Closed
 
 let set_election_auto_dates uuid d =
-  let e_auto_open = Option.map datetime_of_unixfloat d.auto_date_open in
-  let e_auto_close = Option.map datetime_of_unixfloat d.auto_date_close in
+  let e_auto_open = Option.map Datetime.from_unixfloat d.auto_date_open in
+  let e_auto_close = Option.map Datetime.from_unixfloat d.auto_date_close in
   let* dates = Web_persist.get_election_dates uuid in
   Web_persist.set_election_dates uuid {dates with e_auto_open; e_auto_close}
 
-let perform_server_side_decryption election metadata tally =
-  let module W = (val election : Site_common_sig.ELECTION_LWT) in
-  let uuid = W.election.e_uuid in
-  let tally = encrypted_tally_of_string W.G.read tally in
-  let decrypt i =
-    let* x = Web_persist.get_private_key uuid in
-    match x with
-    | Some sk ->
-       let* pd = W.E.compute_factor tally sk in
-       let pd = string_of_partial_decryption W.G.write pd in
-       Web_persist.add_partial_decryption uuid (i, pd)
-    | None ->
-       Lwt.fail (Error "missing server private key")
-  in
-  Option.value metadata.e_trustees ~default:["server"]
-  |> List.mapi (fun i t -> i, t)
-  |> Lwt_list.iter_s
-       (fun (i, t) ->
-         if t = "server" then decrypt (i + 1) else Lwt.return_unit
-       )
-
 let get_partial_decryptions uuid metadata =
   let@ () = fun cont ->
     let* state = Web_persist.get_election_state uuid in
     match state with
-    | `EncryptedTally _ -> cont ()
+    | `EncryptedTally -> cont ()
     | _ -> Lwt.fail @@ Error "not in state EncryptedTally"
   in
   let open Belenios_core.Serializable_j in
@@ -225,7 +197,7 @@ let get_partial_decryptions uuid metadata =
                  {
                    trustee_pd_address = Option.value name ~default:"";
                    trustee_pd_token = token;
-                   trustee_pd_done = List.mem_assoc id pds;
+                   trustee_pd_done = List.exists (fun x -> x.owned_owner = id) pds;
                  }
                )
         end;
@@ -233,121 +205,44 @@ let get_partial_decryptions uuid metadata =
     }
 
 
-let transition_to_encrypted_tally election metadata tally =
-  let module W = (val election : Site_common_sig.ELECTION_LWT) in
-  let uuid = W.election.e_uuid in
-  let* () = perform_server_side_decryption election metadata tally in
-  Web_persist.set_election_state uuid @@ `EncryptedTally (0, 0, "")
+let transition_to_encrypted_tally uuid =
+  Web_persist.set_election_state uuid `EncryptedTally
 
-let compute_encrypted_tally election metadata =
+let compute_encrypted_tally election =
   let module W = (val election : Site_common_sig.ELECTION_LWT) in
   let uuid = W.election.e_uuid in
   let* state = Web_persist.get_election_state uuid in
   match state with
   | `Closed ->
-     let* tally = Web_persist.compute_encrypted_tally election in
+     let* () = Web_persist.compute_encrypted_tally election in
      if Belenios.Election.has_nh_questions W.election then (
        let* () = Web_persist.set_election_state uuid `Shuffling in
        Lwt.return_true
      ) else (
-       let* () = transition_to_encrypted_tally election metadata tally in
+       let* () = transition_to_encrypted_tally uuid in
        Lwt.return_true
      )
   | _ -> Lwt.return_false
 
-let finish_shuffling election metadata =
+let finish_shuffling election =
   let module W = (val election : Site_common_sig.ELECTION_LWT) in
   let uuid = W.election.e_uuid in
   let* state = Web_persist.get_election_state uuid in
   match state with
   | `Shuffling ->
-     begin
-       let* x = Web_persist.compute_encrypted_tally_after_shuffling election in
-       match x with
-       | None -> Lwt.fail (Error "compute_encrypted_tally_after_shuffling")
-       | Some tally ->
-          let* () = transition_to_encrypted_tally election metadata tally in
-          Lwt.return_true
-     end
+     let* () = Web_events.append ~uuid [Event (`EndShuffles, None)] in
+     let* () = Spool.del ~uuid Spool.skipped_shufflers in
+     let* () = transition_to_encrypted_tally uuid in
+     Lwt.return_true
   | _ -> Lwt.return_false
 
-let release_tally election =
-  let module W = (val election : Site_common_sig.ELECTION_LWT) in
-  let uuid = W.election.e_uuid in
-  let@ () = fun cont ->
-    let* state = Web_persist.get_election_state uuid in
-    match state with
-    | `EncryptedTally _ -> cont ()
-    | _ -> Lwt.return @@ Stdlib.Error `Forbidden
-  in
-  let* ntallied =
-    let* hashes = Web_persist.get_ballot_hashes uuid in
-    let weights = List.map snd hashes in
-    let open Weight in
-    Lwt.return @@ List.fold_left ( + ) zero weights
-  in
-  let@ et = fun cont ->
-    let* x = read_file ~uuid (string_of_election_file ESETally) in
-    match x with
-    | Some [et] ->
-       begin
-         match Option.wrap (encrypted_tally_of_string W.G.read) et with
-         | None -> Lwt.fail (Error "release_tally: encrypted_tally_of_string")
-         | Some x -> cont x
-       end
-    | _ -> Lwt.fail (Error "release_tally")
-  in
-  let open Belenios_core.Serializable_j in
-  let* trustees = Web_persist.get_trustees uuid in
-  let trustees = trustees_of_string W.G.read trustees in
-  let* pds = Web_persist.get_partial_decryptions uuid in
-  let pds = List.map snd pds in
-  let pds = List.map (partial_decryption_of_string W.G.read) pds in
-  let* shuffles, shufflers =
-    let* x = Web_persist.get_shuffles uuid in
-    match x with
-    | None -> Lwt.return (None, None)
-    | Some s ->
-       let s = List.map (shuffle_of_string W.G.read) s in
-       let* x = Web_persist.get_shuffle_hashes uuid in
-       match x with
-       | None -> Lwt.return (Some s, None)
-       | Some x ->
-          let x =
-            x
-            |> List.map (fun x -> if x.sh_hash = "" then [] else [x.sh_name])
-            |> List.flatten
-          in
-          assert (List.length s = List.length x);
-          Lwt.return (Some s, Some x)
-  in
-  match W.E.compute_result ?shuffles ?shufflers ntallied et pds trustees with
-  | Ok result ->
-     let* () =
-       let result =
-         string_of_election_result
-           W.write_result (write_encrypted_tally W.G.write)
-           (write_partial_decryption W.G.write) (write_shuffle W.G.write)
-           result
-       in
-       write_file ~uuid (string_of_election_file ESResult) [result]
-     in
-     let* () = Web_persist.remove_audit_cache uuid in
-     let* () = Web_persist.set_election_state uuid `Tallied in
-     let* dates = Web_persist.get_election_dates uuid in
-     let* () = Web_persist.set_election_dates uuid {dates with e_tally = Some (now ())} in
-     let* () = cleanup_file (uuid /// "decryption_tokens.json") in
-     let* () = cleanup_file (uuid /// "shuffles.jsons") in
-     let* () = Web_persist.clear_shuffle_token uuid in
-     Lwt.return @@ Ok ()
-  | Error e -> Lwt.return @@ Stdlib.Error (`CombinationError e)
-
 let delete_sensitive_data uuid =
   let* () = cleanup_file (uuid /// "state.json") in
   let* () = cleanup_file (uuid /// "decryption_tokens.json") in
   let* () = cleanup_file (uuid /// "partial_decryptions.json") in
   let* () = cleanup_file (uuid /// "extended_records.jsons") in
   let* () = cleanup_file (uuid /// "credential_mappings.jsons") in
+  let* () = cleanup_file (uuid /// "public_creds.json") in
   let* () = rmdir (uuid /// "ballots") in
   let* () = cleanup_file (uuid /// "ballots_index.json") in
   let* () = cleanup_file (uuid /// "private_key.json") in
@@ -357,7 +252,7 @@ let delete_sensitive_data uuid =
 let archive_election uuid =
   let* () = delete_sensitive_data uuid in
   let* dates = Web_persist.get_election_dates uuid in
-  Web_persist.set_election_dates uuid {dates with e_archive = Some (now ())}
+  Web_persist.set_election_dates uuid {dates with e_archive = Some (Datetime.now ())}
 
 let delete_election election metadata =
   let module W = (val election : Site_common_sig.ELECTION_LWT) in
@@ -372,7 +267,7 @@ let delete_election election metadata =
       t_credential_authority = None;
     }
   in
-  let de_owner = Option.value metadata.e_owner ~default:(`Id []) in
+  let de_owners = metadata.e_owners in
   let* dates = Web_persist.get_election_dates uuid in
   let de_date =
     match dates.e_tally with
@@ -424,7 +319,7 @@ let delete_election election metadata =
   let de = {
       de_uuid = uuid;
       de_template;
-      de_owner;
+      de_owners;
       de_nb_voters;
       de_nb_ballots = List.length ballots;
       de_date;
@@ -437,22 +332,17 @@ let delete_election election metadata =
   in
   let* () = write_file ~uuid "deleted.json" [string_of_deleted_election de] in
   let files_to_delete = [
-      "election.json";
-      "ballots.jsons";
+      Uuid.unwrap uuid ^ ".bel";
       "dates.json";
-      "encrypted_tally.json";
       "metadata.json";
       "passwords.csv";
-      "public_creds.txt";
-      "trustees.json";
       "records";
-      "result.json";
       "hide_result";
       "shuffle_token";
-      "shuffles.jsons";
       "voters.txt";
       "archive.zip";
       "audit_cache.json";
+      Spool.filename Spool.skipped_shufflers;
     ]
   in
   let* () =
@@ -519,9 +409,9 @@ let set_postpone_date uuid date =
     match date with
     | None -> cont None
     | Some t ->
-       let t = datetime_of_unixfloat t in
-       let max = datetime_add (now ()) (day days_to_publish_result) in
-       if datetime_compare t max > 0 then
+       let t = Datetime.from_unixfloat t in
+       let max = Period.add (Datetime.now ()) (Period.day days_to_publish_result) in
+       if Datetime.compare t max > 0 then
          Lwt.return_false
        else
          cont (Some t)
@@ -536,21 +426,29 @@ let get_shuffles uuid metadata =
     | `Shuffling -> cont ()
     | _ -> Lwt.fail @@ Error "not in state Shuffling"
   in
-  let* hashes = Web_persist.get_shuffle_hashes uuid in
-  let hashes = Option.value hashes ~default:[] in
+  let* shuffles = Web_persist.get_shuffles uuid in
+  let shuffles = Option.value shuffles ~default:[] in
+  let* skipped = Spool.get ~uuid Spool.skipped_shufflers in
+  let skipped = Option.value skipped ~default:[] in
   let* token = Web_persist.get_shuffle_token uuid in
   Lwt.return {
       shuffles_shufflers =
         begin
           (match metadata.e_trustees with None -> ["server"] | Some ts -> ts)
-          |> List.map
-               (fun t ->
+          |> List.mapi
+               (fun i t ->
+                 let trustee_id = i + 1 in
                  {
                    shuffler_address = t;
                    shuffler_fingerprint =
-                     hashes
-                     |> List.find_opt (fun x -> x.sh_trustee = t)
-                     |> Option.map (fun x -> x.sh_hash);
+                     List.find_map
+                       (fun (_, o, _) ->
+                         if o.owned_owner = trustee_id then
+                           Some (Hash.to_b64 o.owned_payload)
+                         else if List.mem t skipped then
+                           Some ""
+                         else None
+                       ) shuffles;
                    shuffler_token =
                      Option.bind token
                        (fun x -> if x.tk_trustee = t then Some x.tk_token else None);
@@ -571,6 +469,7 @@ let extract_names trustees =
         | `Single x -> [x.trustee_name]
        )
   |> List.flatten
+  |> List.mapi (fun i x -> i + 1, x)
 
 let get_trustee_names uuid =
   let open Belenios_core.Serializable_j in
@@ -579,21 +478,32 @@ let get_trustee_names uuid =
   Lwt.return (extract_names trustees)
 
 let get_trustee_name uuid metadata trustee =
-  let&* xs = metadata.e_trustees in
-  let* names = get_trustee_names uuid in
-  Lwt.return (List.assoc trustee (List.combine xs names))
+  match metadata.e_trustees with
+  | None -> Lwt.return (1, None)
+  | Some xs ->
+     let* names = get_trustee_names uuid in
+     Lwt.return (List.assoc trustee (List.combine xs names))
 
-let skip_shuffler uuid metadata trustee =
-  let* sh_name = get_trustee_name uuid metadata trustee in
-  let* () = Web_persist.clear_shuffle_token uuid in
-  let sh = {sh_trustee = trustee; sh_hash = ""; sh_name} in
-  let* () = Web_persist.add_shuffle_hash uuid sh in
-  Lwt.return_unit
+let skip_shuffler uuid trustee =
+  let* x = Web_persist.get_shuffle_token uuid in
+  let* () =
+    match x with
+    | Some x when x.tk_trustee = trustee ->
+       Web_persist.clear_shuffle_token uuid
+    | None -> Lwt.return_unit
+    | _ -> Lwt.fail @@ Error "trustee is not currently selected"
+  in
+  let* x = Spool.get ~uuid Spool.skipped_shufflers in
+  let x = Option.value x ~default:[] in
+  if List.mem trustee x then
+    Lwt.fail @@ Error "trustee already skipped"
+  else
+    Spool.set ~uuid Spool.skipped_shufflers (trustee :: x)
 
 let select_shuffler uuid metadata trustee =
-  let* name = get_trustee_name uuid metadata trustee in
+  let* trustee_id, name = get_trustee_name uuid metadata trustee in
   let* () = Web_persist.clear_shuffle_token uuid in
-  let* _ = Web_persist.gen_shuffle_token uuid trustee name in
+  let* _ = Web_persist.gen_shuffle_token uuid trustee trustee_id name in
   Lwt.return_unit
 
 let split_voting_record =
@@ -601,7 +511,7 @@ let split_voting_record =
   fun x ->
   let s = Pcre.exec ~rex x in
   {
-    vr_date = unixfloat_of_datetime @@ raw_datetime_of_string @@ Pcre.get_substring s 1;
+    vr_date = Datetime.to_unixfloat @@ Datetime.wrap @@ Pcre.get_substring s 1;
     vr_username = Pcre.get_substring s 3;
   }
 
@@ -610,6 +520,47 @@ let get_records uuid =
   let x = Option.value x ~default:[] in
   Lwt.return @@ List.map split_voting_record x
 
+let cast_ballot send_confirmation election ~rawballot ~user =
+  let module W = (val election : Site_common_sig.ELECTION_LWT) in
+  let uuid = W.election.e_uuid in
+  let* voters = read_file ~uuid "voters.txt" in
+  let voters = match voters with Some xs -> xs | None -> [] in
+  let* email, login, weight =
+    let rec loop = function
+      | x :: xs ->
+         let email, login, weight = split_identity x in
+         if String.lowercase_ascii login = String.lowercase_ascii user.user_name then Lwt.return (email, login, weight) else loop xs
+      | [] -> fail UnauthorizedVoter
+    in loop voters
+  in
+  let show_weight =
+    List.exists
+      (fun x ->
+        let _, _, weight = split_identity_opt x in
+        weight <> None
+      ) voters
+  in
+  let oweight = if show_weight then Some weight else None in
+  let user_s = string_of_user user in
+  let* state = Web_persist.get_election_state uuid in
+  let voting_open = state = `Open in
+  let* () = if not voting_open then fail ElectionClosed else Lwt.return_unit in
+  let* r = Web_persist.cast_ballot election ~rawballot ~user:user_s ~weight (Datetime.now ()) in
+  match r with
+  | Ok (hash, revote) ->
+     let* success = send_confirmation uuid revote login email oweight hash in
+     let () =
+       if revote then
+         Printf.ksprintf Ocsigen_messages.accesslog
+           "Someone revoted in election %s" (Uuid.unwrap uuid)
+     in
+     Lwt.return (user, hash, revote, weight, success)
+  | Error e ->
+     fail (CastError e)
+
+let direct_voter_auth =
+  ref (fun _ _ -> assert false) (* initialized in Web_main *)
+
 let dispatch_election ~token ~ifmatch endpoint method_ body uuid raw metadata =
   match endpoint with
   | [] ->
@@ -642,23 +593,12 @@ let dispatch_election ~token ~ifmatch endpoint method_ body uuid raw metadata =
                  | `FinishShuffling -> finish_shuffling
                in
                let module W = Belenios.Election.Make (struct let raw_election = raw end) (LwtRandom) () in
-               let* b = doit (module W) metadata in
+               let* b = doit (module W) in
                if b then ok else forbidden
             | `ReleaseTally ->
-               begin
-                 let@ () = handle_generic_error in
-                 let module W = Belenios.Election.Make (struct let raw_election = raw end) (LwtRandom) () in
-                 let* b = release_tally (module W) in
-                 match b with
-                 | Ok () -> ok
-                 | Error `Forbidden -> forbidden
-                 | Error (`CombinationError e) ->
-                    let msg =
-                      Printf.sprintf "combination error: %s"
-                        (Belenios.Trustees.string_of_combination_error e)
-                    in
-                    Lwt.fail @@ Error msg
-               end
+               let@ () = handle_generic_error in
+               let* () = Web_persist.release_tally uuid in
+               ok
             | `Archive ->
                let@ () = handle_generic_error in
                let* () = archive_election uuid in
@@ -764,11 +704,39 @@ let dispatch_election ~token ~ifmatch endpoint method_ body uuid raw metadata =
             let@ request = body.run shuffler_request_of_string in
             let@ () = handle_generic_error in
             match request with
-            | `Skip -> let* () = skip_shuffler uuid metadata shuffler in ok
+            | `Skip -> let* () = skip_shuffler uuid shuffler in ok
             | `Select -> let* () = select_shuffler uuid metadata shuffler in ok
           end
        | _ -> method_not_allowed
      end
+  | ["ballots"] ->
+     begin
+       match method_ with
+       | `POST ->
+          begin
+            let@ token = Option.unwrap unauthorized token in
+            let@ user = fun cont ->
+              Lwt.catch
+                (fun () ->
+                  let json =
+                    match Base64.decode token with
+                    | Ok x -> Yojson.Safe.from_string x
+                    | Error (`Msg x) -> failwith x
+                  in
+                  let* x = !direct_voter_auth uuid json in
+                  cont x
+                )
+                (fun _ -> unauthorized)
+            in
+            let@ rawballot = body.run Fun.id in
+            let@ () = handle_generic_error in
+            let send_confirmation _ _ _ _ _ _ = Lwt.return_true in
+            let module W = Belenios.Election.Make (struct let raw_election = raw end) (LwtRandom) () in
+            let* _ = cast_ballot send_confirmation (module W) ~rawballot ~user in
+            ok
+          end
+       | _ -> method_not_allowed
+     end
   | _ -> not_found
 
 let dispatch ~token ~ifmatch endpoint method_ body =
@@ -783,7 +751,7 @@ let dispatch ~token ~ifmatch endpoint method_ body =
           let elections =
             List.fold_left
               (fun accu (kind, summary_uuid, date, summary_name) ->
-                let summary_date = unixfloat_of_datetime date in
+                let summary_date = Datetime.to_unixfloat date in
                 match kind with
                 | `Draft -> accu
                 | (`Validated | `Tallied | `Archived) as x ->
@@ -795,7 +763,7 @@ let dispatch ~token ~ifmatch endpoint method_ body =
        | _ -> method_not_allowed
      end
   | uuid :: endpoint ->
-     let@ uuid = Option.unwrap bad_request (Option.wrap uuid_of_raw_string uuid) in
+     let@ uuid = Option.unwrap bad_request (Option.wrap Uuid.wrap uuid) in
      let* raw = Web_persist.get_raw_election uuid in
      let@ raw = Option.unwrap not_found raw in
      let* metadata = Web_persist.get_election_metadata uuid in
