@@ -21,10 +21,8 @@
 
 open Lwt.Syntax
 open Web_serializable_t
-open Belenios_core.Serializable_builtin_t
 open Belenios_core.Common
 open Belenios_api.Serializable_t
-open Web_serializable_builtin_t
 open Web_common
 
 let ( let& ) = Option.bind
@@ -38,13 +36,13 @@ let tokens = ref SMap.empty
 
 let new_token account =
   let* token = generate_token ~length:22 () in
-  let expiration = datetime_add (now ()) (day 1) in
+  let expiration = Period.add (Datetime.now ()) (Period.day 1) in
   tokens := SMap.add token {expiration; account} !tokens;
   Lwt.return token
 
 let filter tokens =
-  let now = now () in
-  SMap.filter (fun _ {expiration; _} -> datetime_compare now expiration < 0) tokens
+  let now = Datetime.now () in
+  SMap.filter (fun _ {expiration; _} -> Datetime.compare now expiration < 0) tokens
 
 let lookup_token token =
   tokens := filter !tokens;
@@ -110,10 +108,11 @@ let handle_get get =
 
 let get_configuration () =
   {
+    privacy_policy = !Web_config.gdpr_uri;
     belenios_version = Belenios_platform.Version.version;
     belenios_build = Belenios_platform.Version.build;
     spec_version = Belenios_platform.Version.spec;
-    api_version = 1;
+    api_version = 2;
     supported_crypto_versions;
     supported_booth_versions;
     authentications =
