@@ -23,7 +23,6 @@ open Lwt
 open Lwt.Syntax
 open Belenios_platform
 open Belenios_core
-open Serializable_builtin_t
 open Common
 open Web_common
 open Eliom_content.Html.F
@@ -47,18 +46,19 @@ module Make (Web_i18n : Web_i18n_sig.S) (Web_services : Web_services_sig.S) = st
     let href = uri_of_string (fun () -> Eliom_uri.make_string_uri ~service x) in
     Eliom_content.Html.F.Raw.a ~a:(a_href href :: a) contents
 
+  let absolute_uri_of_service ~service x =
+    Eliom_uri.make_string_uri ~absolute:true ~service x
+    |> rewrite_prefix
+    |> Eliom_content.Xml.uri_of_string
+
   let static x =
     let service =
       Eliom_service.static_dir_with_params
         ~get_params:(Eliom_parameter.string "version") ()
     in
-    make_uri ~service (["static"; x], Version.build)
-
-  let belenios_url = Eliom_service.extern
-                       ~prefix:"https://www.belenios.org"
-                       ~path:[]
-                       ~meth:(Eliom_service.Get Eliom_parameter.unit)
-                       ()
+    Eliom_uri.make_string_uri ~absolute:true ~service (["static"; x], Version.build)
+    |> rewrite_prefix
+    |> Eliom_content.Xml.uri_of_string
 
   let get_preferred_gettext () = Web_i18n.get_preferred_gettext "voter"
 
@@ -75,7 +75,23 @@ module Make (Web_i18n : Web_i18n_sig.S) (Web_services : Web_services_sig.S) = st
        | None -> return default
        | Some x -> return @@ Unsafe.data (String.concat "\n" x)
 
-  let base ~title ?login_box ?lang_box ~content ?(footer = txt "") ?uuid ?static:(static_page = false) () =
+  module UiBase = struct
+    module Xml = Eliom_content.Xml
+    module Svg = Eliom_content.Svg.F.Raw
+    module Html = Eliom_content.Html.F.Raw
+
+    module Uris = struct
+      let home = absolute_uri_of_service ~service:home ()
+      let logo = absolute_uri_of_service ~service:logo ()
+      let belenios = Eliom_content.Xml.uri_of_string Belenios_ui.Links.belenios
+      let source_code = absolute_uri_of_service ~service:source_code ()
+      let privacy_policy = Xml.uri_of_string !Web_config.gdpr_uri
+    end
+  end
+
+  module Ui = Belenios_ui.Pages_common.Make(UiBase)
+
+  let base ~title ?full_title ?(login_box = txt "") ?lang_box ~content ?(footer = txt "") ?uuid ?static:(static_page = false) () =
     let* l = get_preferred_gettext () in
     let open (val l) in
     let administer =
@@ -83,15 +99,7 @@ module Make (Web_i18n : Web_i18n_sig.S) (Web_services : Web_services_sig.S) = st
       | None ->
          raw_a ~service:admin [txt (s_ "Administer elections")] ()
       | Some uuid ->
-         raw_a ~service:election_admin ~a:[a_id ("election_admin_" ^ (raw_string_of_uuid uuid))] [txt (s_ "Administer this election")] uuid
-    in
-    let login_box = match login_box with
-      | None ->
-         div ~a:[a_style "float: right; padding: 10px;"] [
-             img ~a:[a_height 70] ~alt:""
-               ~src:(static "placeholder.png") ();
-           ]
-      | Some x -> x
+         raw_a ~service:election_admin ~a:[a_id ("election_admin_" ^ (Uuid.unwrap uuid))] [txt (s_ "Administer this election")] uuid
     in
     let lang_box =
       match lang_box with
@@ -101,127 +109,19 @@ module Make (Web_i18n : Web_i18n_sig.S) (Web_services : Web_services_sig.S) = st
     let maybe_static x = if static_page then Lwt.return @@ txt "" else read_snippet ~lang x in
     let* warning = maybe_static !Web_config.warning_file in
     let* extra_footer = maybe_static !Web_config.footer_file in
-    Lwt.return (html ~a:[a_dir `Ltr; a_xml_lang lang]
-                  (head (Eliom_content.Html.F.title (txt title)) [
-                       script (txt "window.onbeforeunload = function () {};");
-                       link ~rel:[`Stylesheet] ~href:(static "site.css") ();
-                  ])
-                  (body [
-                       div ~a:[a_id "wrapper"] [
-                           div ~a:[a_id "header"] [
-                               div [
-                                   div ~a:[a_style "float: left; padding: 10px;"] [
-                                       raw_a ~service:home [
-                                           img ~alt:(s_ "Election server") ~a:[a_height 70]
-                                             ~src:(static "logo.png") ();
-                                         ] ();
-                                     ];
-                                   login_box;
-                                   h1 ~a:[a_style "text-align: center; padding: 20px;"] [txt title];
-                                   div ~a:[a_style "clear: both;"] [];
-                                 ];
-                             ];
-                           warning;
-                           div ~a:[a_id "main"]
-                             [
-                               lang_box;
-                               div content;
-                             ];
-                           div ~a:[a_id "footer"; a_style "text-align: center;" ] [
-                               div ~a:[a_id "bottom"] [
-                                   footer;
-                                   txt (s_ "Powered by ");
-                                   raw_a ~service:belenios_url [txt "Belenios"] ();
-                                   Version.(
-                                     Printf.ksprintf txt " %s (%s). " version build
-                                   );
-                                   raw_a ~service:source_code [txt (s_ "Get the source code")] ();
-                                   txt ". ";
-                                   direct_a !Web_config.gdpr_uri (s_ "Privacy policy");
-                                   txt ". ";
-                                   administer;
-                                   txt ".";
-                                   extra_footer;
-                                 ]
-                         ]]
-      ]))
-
-  let responsive_base ~title ?full_title ?login_box ?lang_box ~content ?(footer = txt "") ?uuid () =
-    let* l = get_preferred_gettext () in
-    let open (val l) in
-    let administer =
-      match uuid with
-      | None ->
-         a ~service:admin [txt (s_ "Administer elections")] ()
-      | Some uuid ->
-         a ~service:election_admin ~a:[a_id ("election_admin_" ^ (raw_string_of_uuid uuid))] [txt (s_ "Administer this election")] uuid
-    in
-    let login_box = match login_box with
-      | None ->
-         div []
-      | Some x -> x
-    in
-    let lang_box =
-      match lang_box with
-      | None -> div []
-      | Some x -> div [x; div ~a:[a_style "clear: both;"] []]
-    in
-    let* warning = read_snippet ~lang !Web_config.warning_file in
-    let* extra_footer = read_snippet ~lang !Web_config.footer_file in
     let full_title =
       match full_title with
       | None -> [txt title]
       | Some x -> txt_br x
     in
-    Lwt.return (html ~a:[a_dir `Ltr; a_xml_lang lang]
-                  (head (Eliom_content.Html.F.title (txt title)) [
-                       meta ~a:[a_name "viewport"; a_content "width=device-width, initial-scale=1"] ();
-                       script (txt "window.onbeforeunload = function () {};");
-                       link ~rel:[`Stylesheet] ~href:(static "responsive_site.css") ();
-                  ])
-                  (body [
-                       div ~a:[a_id "vote-app"] [
-                           div ~a:[a_class ["page"]] [
-                               div ~a:[a_id "header"; a_class ["page-header"]] [
-                                   div ~a:[a_class ["page-header__logo"]] [
-                                       a ~service:home [
-                                           img ~a:[a_class ["page-header__logo__image"]] ~alt:(s_ "Election server")
-                                             ~src:(static "logo.png") ();
-                                         ] ();
-                                     ];
-                                   div ~a:[a_class ["page-header__titles"]] [
-                                       h1 ~a:[a_class ["page-header__titles__election-name"]; a_id "election_name"] full_title;
-                                       p ~a:[a_class ["page-header__titles__election-description"]; a_id "election_description"] [txt ""]; (* no description provided? *)
-                                     ];
-                                   div ~a:[a_class ["page-header__right"]] [
-                                       login_box;
-                                     ];
-                                 ];
-                               div ~a:[a_class ["page-body"]] [
-                                   warning;
-                                   div ~a:[a_id "main"] [
-                                       lang_box;
-                                       div content;
-                                     ]
-                                 ];
-                               div ~a:[a_class ["page-footer"]] [
-                                   footer;
-                                   txt (s_ "Powered by ");
-                                   a ~service:belenios_url [txt "Belenios"] ();
-                                   Version.(
-                                     Printf.ksprintf txt " %s (%s). " version build
-                                   );
-                                   a ~service:source_code [txt (s_ "Get the source code")] ();
-                                   txt ". ";
-                                   direct_a !Web_config.gdpr_uri (s_ "Privacy policy");
-                                   txt ". ";
-                                   administer;
-                                   txt ".";
-                                   extra_footer;
-                                 ];
-                             ];
-                         ];
-                  ])
+    Lwt.return
+      (html ~a:[a_dir `Ltr; a_xml_lang lang]
+         (head (Eliom_content.Html.F.title (txt title)) [
+              meta ~a:[a_name "viewport"; a_content "width=device-width, initial-scale=1"] ();
+              script (txt "window.onbeforeunload = function () {};");
+              link ~rel:[`Stylesheet] ~href:(static "responsive_site.css") ();
+         ])
+         (body (Ui.base_body l ~full_title ~login_box ~warning ~lang_box ~content ~footer ~administer ~extra_footer ()))
       )
 
   let lang_box cont =
@@ -246,7 +146,7 @@ module Make (Web_i18n : Web_i18n_sig.S) (Web_services : Web_services_sig.S) = st
                   div ~a:[a_style "font-size: 80%; font-style: italic; text-align: right;"]
                     [
                       txt "(";
-                      direct_a "https://www.belenios.org/translation.html" (s_ "Wish to help with translations?");
+                      direct_a Belenios_ui.Links.translation (s_ "Wish to help with translations?");
                       txt ")";
                     ];
                 ]
@@ -288,7 +188,7 @@ module Make (Web_i18n : Web_i18n_sig.S) (Web_services : Web_services_sig.S) = st
         p [txt message];
         proceed;
       ] in
-    responsive_base ~title ~content ()
+    base ~title ~content ()
 
   let raw_textarea ?rows ?cols id contents =
     let id = [a_id id] in
@@ -326,7 +226,7 @@ module Make (Web_i18n : Web_i18n_sig.S) (Web_services : Web_services_sig.S) = st
                  [txt (s_ "Please log in:"); txt " ["] @ auth_systems @ [txt "]"]
           )]
       ] in
-    responsive_base ~title:(s_ "Log in") ~content ()
+    base ~title:(s_ "Log in") ~content ()
 
   let login_generic site_or_election username_or_address ~service ~state =
     let* l = get_preferred_gettext () in
@@ -421,7 +321,7 @@ module Make (Web_i18n : Web_i18n_sig.S) (Web_services : Web_services_sig.S) = st
           ];
       ]
     in
-    responsive_base ~title ~content ()
+    base ~title ~content ()
 
   let email_login site_or_election =
     let* l = get_preferred_gettext () in
