@@ -22,10 +22,8 @@
 open Lwt
 open Lwt.Syntax
 open Belenios_core
-open Serializable_builtin_t
 open Serializable_j
 open Common
-open Web_serializable_builtin_t
 open Web_serializable_j
 open Web_common
 open Eliom_content.Html.F
@@ -57,14 +55,8 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
                               txt (s_ "parameters")
                             ] ();
                           txt ", ";
-                          a ~service:(file uuid ESTrustees) [txt (s_ "trustees")] ();
-                          txt ", ";
-                          a ~service:(file uuid ESCreds) [
-                              txt (s_ "public credentials")
-                            ] ();
-                          txt ", ";
-                          a ~service:(file uuid ESBallots) [
-                              txt (s_ "ballots")
+                          a ~service:(file uuid (ESArchive uuid)) [
+                              txt (s_ "public data")
                             ] ();
                           txt ".";
                         ];
@@ -171,7 +163,7 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
   let format_question_result uuid l (i, q) r =
     let open (val l : Belenios_ui.I18n.GETTEXT) in
     match q, r with
-    | Question.Homomorphic x, RHomomorphic r ->
+    | Question.Homomorphic x, `Homomorphic r ->
        let open Question_h_t in
        let answers = Array.to_list x.q_answers in
        let answers = match x.q_blank with
@@ -195,7 +187,7 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
            div ~a:[a_class ["result_question"]] (txt_br x.q_question);
            answers;
          ]
-    | Question.NonHomomorphic (q, extra), RNonHomomorphic ballots ->
+    | Question.NonHomomorphic (q, extra), `NonHomomorphic ballots ->
        let open Question_nh_t in
        let applied_counting_method, show_others =
          match Question.get_counting_method extra with
@@ -251,17 +243,17 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
     let uuid = params.e_uuid in
     let* metadata = Web_persist.get_election_metadata uuid in
     let* dates = Web_persist.get_election_dates uuid in
-    let now = now () in
+    let now = Datetime.now () in
     let state_ =
       match state with
       | `Closed ->
          let it_will_open =
            match dates.e_auto_open with
-           | Some t when datetime_compare now t < 0 ->
+           | Some t when Datetime.compare now t < 0 ->
               span [
                   txt " ";
                   txt (s_ "It will open in ");
-                  txt (format_period l (datetime_sub t now));
+                  txt (format_period l (Period.sub t now));
                   txt ".";
                 ]
            | _ -> txt ""
@@ -274,10 +266,10 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
       | `Open ->
          let it_will_close =
            match dates.e_auto_close with
-           | Some t when datetime_compare now t < 0 ->
+           | Some t when Datetime.compare now t < 0 ->
               span [
                   txt (s_ "The election will close in ");
-                  txt (format_period l (datetime_sub t now));
+                  txt (format_period l (Period.sub t now));
                   txt ".";
                 ]
            | _ -> txt ""
@@ -288,7 +280,7 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
            txt " ";
            b [txt (s_ "The election is closed and being tallied.")];
          ]
-      | `EncryptedTally _ ->
+      | `EncryptedTally ->
          [
            txt " ";
            b [txt (s_ "The election is closed and being tallied.")];
@@ -325,7 +317,7 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
         let a =
           a_id "start"
           :: a_user_data "uri" uri
-          :: a_user_data "uuid" (raw_string_of_uuid uuid)
+          :: a_user_data "uuid" (Uuid.unwrap uuid)
           :: a_user_data "lang" lang
           :: a_style "font-size:35px;"
           :: disabled
@@ -343,31 +335,31 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
     in
     let* middle =
       let* result = Web_persist.get_election_result uuid in
-      let result =
-        Option.map (
-            election_result_of_string
-              W.read_result Yojson.Safe.read_json
-              Yojson.Safe.read_json Yojson.Safe.read_json
-          ) result
-      in
+      let result = Option.map (election_result_of_string W.read_result) result in
       let* hidden = Web_persist.get_election_result_hidden uuid in
       let* is_admin =
         let* metadata = Web_persist.get_election_metadata uuid in
         let* site_user = Eliom_reference.get Web_state.site_user in
-        match site_user, metadata.e_owner with
-        | Some x, Some y -> return @@ Accounts.check x y
+        match site_user with
+        | Some (_, a, _) -> return @@ Accounts.check a metadata.e_owners
         | _ -> return_false
       in
       match result with
       | Some r when hidden = None || is_admin ->
-         let* hashes = Web_persist.get_ballot_hashes uuid in
-         let nballots = List.length hashes in
+         let* nballots, total_weight =
+           let* x = Web_persist.get_sized_encrypted_tally uuid in
+           match x with
+           | None -> assert false
+           | Some x ->
+              let x = sized_encrypted_tally_of_string read_hash x in
+              Lwt.return (x.sized_num_tallied, x.sized_total_weight)
+         in
          let div_total_weight =
-           if not Weight.(is_int r.num_tallied nballots) then (
+           if not Weight.(is_int total_weight nballots) then (
              div [
                  txt (s_ "Total weight of accepted ballots:");
                  txt " ";
-                 txt (Weight.to_string r.num_tallied);
+                 txt (Weight.to_string total_weight);
                ]
            ) else (
              txt ""
@@ -386,7 +378,7 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
                        div [
                            txt (s_ "You can also download the ");
                            a ~service:election_dir
-                             [txt (s_ "result with cryptographic proofs")]
+                             [txt (s_ "raw result")]
                              (uuid, ESResult);
                            txt ".";
                          ];
@@ -401,7 +393,7 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
            div [
                Printf.ksprintf txt
                  (f_ "The result of this election is currently not publicly available. It will be in %s.")
-                 (format_period l (datetime_sub t now));
+                 (format_period l (Period.sub t now));
              ]
       | None -> return go_to_the_booth
     in
@@ -552,7 +544,7 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
     let* lang_box = lang_box (ContSiteElection uuid) in
     let title = br_truncate params.e_name in
     let full_title = params.e_name in
-    responsive_base ~lang_box ~full_title ~title ~content ~footer ~uuid ()
+    base ~lang_box ~full_title ~title ~content ~footer ~uuid ()
 
   let cast_raw election () =
     let* l = get_preferred_gettext () in
@@ -585,7 +577,7 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
       let hash =
         Netencoding.Url.mk_url_encoded_parameters
           [
-            "uuid", raw_string_of_uuid uuid;
+            "uuid", Uuid.unwrap uuid;
             "lang", lang;
           ]
       in
@@ -635,143 +627,58 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
     let* footer = audit_footer election in
     let title = br_truncate params.e_name in
     let full_title = params.e_name in
-    responsive_base ~full_title ~title ~content ~uuid ~footer ()
+    base ~full_title ~title ~content ~uuid ~footer ()
 
-  let cast_confirmation election hash () =
-    let* l = get_preferred_gettext () in
-    let open (val l) in
-    let module W = (val election : Site_common_sig.ELECTION_LWT) in
-    let open W in
-    let uuid = election.e_uuid in
-    let* user = Web_state.get_election_user uuid in
-    let name = election.e_name in
-    let user_div = match user with
-      | Some u ->
-         post_form ~service:election_cast_confirm (fun () -> [
-                                                       p ~a:[a_style "text-align: center; padding: 10px;"] [
-                                                           txt (s_ "I am ");
-                                                           em [txt u.user_name];
-                                                           txt (s_ " and ");
-                                                           input
-                                                             ~a:[a_class ["nice-button nice-button--blue"]; a_style "font-size: 18px;"]
-                                                             ~input_type:`Submit ~value:(s_ "I cast my vote") string;
-                                                           txt ".";
-                                                         ]
-           ]) uuid
-      | None ->
-         div [
-             txt (s_ "Please log in to confirm your vote.");
-           ]
-    in
-    let* div_revote =
-      match user with
-      | None -> return @@ txt ""
-      | Some u ->
-         let* revote = Web_persist.has_voted uuid u in
-         if revote then
-           return @@ p [b [txt (s_ "Note: You have already voted. Your vote will be replaced.")]]
-         else
-           return @@ txt ""
-    in
-    let progress_responsive = div ~a:[a_class ["breadcrumb"]; a_style "padding-top: 0;"] [
-                                  div ~a:[a_class ["breadcrumb__step-separator"]] [];
-                                  div ~a:[a_class ["breadcrumb__step"]] [
-                                      span ~a:[a_class ["breadcrumb__step__title"]] [
-                                          txt (s_ "Input credential");
-                                        ];
-                                      span ~a:[a_class ["breadcrumb__step__short-title"]; a_title (s_ "Input credential")] [
-                                          txt (s_ "Step 1");
-                                        ];
-                                    ];
-                                  div ~a:[a_class ["breadcrumb__step-separator"]] [];
-                                  div ~a:[a_class ["breadcrumb__step"]] [
-                                      span ~a:[a_class ["breadcrumb__step__title"]] [
-                                          txt (s_ "Answer to questions");
-                                        ];
-                                      span ~a:[a_class ["breadcrumb__step__short-title"]; a_title (s_ "Answer to questions")] [
-                                          txt (s_ "Step 2");
-                                        ];
-                                    ];
-                                  div ~a:[a_class ["breadcrumb__step-separator"]] [];
-                                  div ~a:[a_class ["breadcrumb__step"]] [
-                                      span ~a:[a_class ["breadcrumb__step__title"]] [
-                                          txt (s_ "Review and encrypt");
-                                        ];
-                                      span ~a:[a_class ["breadcrumb__step__short-title"]; a_title (s_ "Review and encrypt")] [
-                                          txt (s_ "Step 3");
-                                        ];
-                                    ];
-                                  div ~a:[a_class ["breadcrumb__step-separator"]] [];
-                                  div ~a:[a_class ["breadcrumb__step"]] [
-                                      span ~a:[a_class ["breadcrumb__step__title"]] [
-                                          txt (s_ "Authenticate");
-                                        ];
-                                      span ~a:[a_class ["breadcrumb__step__short-title"]; a_title (s_ "Authenticate")] [
-                                          txt (s_ "Step 4");
-                                        ];
-                                    ];
-                                  div ~a:[a_class ["breadcrumb__step-separator"]] [];
-                                  div ~a:[a_class ["breadcrumb__step breadcrumb__step--current"]] [
-                                      span ~a:[a_class ["breadcrumb__step__title"]] [
-                                          txt (s_ "Confirm");
-                                        ];
-                                      span ~a:[a_class ["breadcrumb__step__short-title"]; a_title (s_ "Confirm")] [
-                                          txt (s_ "Step 5");
-                                        ];
-                                    ];
-                                  div ~a:[a_class ["breadcrumb__step-separator"]] [];
-                                ] in
-    let* div_weight =
-      let* audit_cache = Web_persist.get_audit_cache uuid in
-      match audit_cache.cache_checksums.ec_weights with
-      | Some _ ->
-         let* ballot = Eliom_reference.get Web_state.ballot in
-         (match ballot with
-          | Some ballot ->
-             Lwt.catch
-               (fun () ->
-                 let* weight = Web_persist.get_ballot_weight (module W) ballot in
-                 return @@ div [
-                               txt (Printf.sprintf (f_ "Your weight is %s.") (Weight.to_string weight));
-                             ]
-               )
-               (function _ -> return @@ txt "")
-          | None -> return @@ txt ""
-         )
-      | _ -> return @@ txt ""
-    in
-    let content = [
-        progress_responsive;
-        p [
-            txt (s_ "Your ballot for ");
-            em (txt_br name);
-            txt (s_ " has been received, but not recorded yet. ");
-            txt (s_ "Your smart ballot tracker is ");
-            b ~a:[a_id "ballot_tracker"] [
-                txt hash
+  let progress_responsive_step5 l =
+    let open (val l : Belenios_ui.I18n.GETTEXT) in
+    div ~a:[a_class ["breadcrumb"]; a_style "padding-top: 0;"] [
+        div ~a:[a_class ["breadcrumb__step-separator"]] [];
+        div ~a:[a_class ["breadcrumb__step"]] [
+            span ~a:[a_class ["breadcrumb__step__title"]] [
+                txt (s_ "Input credential");
+              ];
+            span ~a:[a_class ["breadcrumb__step__short-title"]; a_title (s_ "Input credential")] [
+                txt (s_ "Step 1");
               ];
-            txt ".";
-            br ();
           ];
-        div_weight;
-        br ();
-        p [txt (s_ "Note: your ballot is encrypted and nobody can see its contents.")];
-        div_revote;
-        user_div;
-        p [
-            (let service =
-               Eliom_service.preapply
-                 ~service:Web_services.election_home (uuid, ())
-             in
-             a ~service [
-                 txt (s_ "Go back to election")
-               ] ());
-            txt ".";
+        div ~a:[a_class ["breadcrumb__step-separator"]] [];
+        div ~a:[a_class ["breadcrumb__step"]] [
+            span ~a:[a_class ["breadcrumb__step__title"]] [
+                txt (s_ "Answer to questions");
+              ];
+            span ~a:[a_class ["breadcrumb__step__short-title"]; a_title (s_ "Answer to questions")] [
+                txt (s_ "Step 2");
+              ];
           ];
-      ] in
-    let title = br_truncate name in
-    let full_title = name in
-    responsive_base ~full_title ~title ~content ~uuid ()
+        div ~a:[a_class ["breadcrumb__step-separator"]] [];
+        div ~a:[a_class ["breadcrumb__step"]] [
+            span ~a:[a_class ["breadcrumb__step__title"]] [
+                txt (s_ "Review and encrypt");
+              ];
+            span ~a:[a_class ["breadcrumb__step__short-title"]; a_title (s_ "Review and encrypt")] [
+                txt (s_ "Step 3");
+              ];
+          ];
+        div ~a:[a_class ["breadcrumb__step-separator"]] [];
+        div ~a:[a_class ["breadcrumb__step"]] [
+            span ~a:[a_class ["breadcrumb__step__title"]] [
+                txt (s_ "Authenticate");
+              ];
+            span ~a:[a_class ["breadcrumb__step__short-title"]; a_title (s_ "Authenticate")] [
+                txt (s_ "Step 4");
+              ];
+          ];
+        div ~a:[a_class ["breadcrumb__step-separator"]] [];
+        div ~a:[a_class ["breadcrumb__step breadcrumb__step--current"]] [
+            span ~a:[a_class ["breadcrumb__step__title"]] [
+                txt (s_ "Confirm");
+              ];
+            span ~a:[a_class ["breadcrumb__step__short-title"]; a_title (s_ "Confirm")] [
+                txt (s_ "Step 5");
+              ];
+          ];
+        div ~a:[a_class ["breadcrumb__step-separator"]] [];
+      ]
 
   let lost_ballot election () =
     let* l = get_preferred_gettext () in
@@ -782,7 +689,7 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
     let uuid = election.e_uuid in
     let* metadata = Web_persist.get_election_metadata uuid in
     let Booth service = fst Web_services.booths.(get_booth_index metadata.e_booth_version) in
-    let hash = Netencoding.Url.mk_url_encoded_parameters ["uuid", raw_string_of_uuid uuid] in
+    let hash = Netencoding.Url.mk_url_encoded_parameters ["uuid", Uuid.unwrap uuid] in
     let content =
       [
         div [
@@ -802,7 +709,7 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
           ];
       ]
     in
-    responsive_base ~full_title ~title ~content ~uuid ()
+    base ~full_title ~title ~content ~uuid ()
 
   let cast_confirmed election ~result () =
     let* l = get_preferred_gettext () in
@@ -810,57 +717,17 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
     let open (val election : Site_common_sig.ELECTION_LWT) in
     let uuid = election.e_uuid in
     let name = election.e_name in
-    let progress_responsive = div ~a:[a_class ["breadcrumb"]; a_style "padding-top: 0;"] [
-                                  div ~a:[a_class ["breadcrumb__step-separator"]] [];
-                                  div ~a:[a_class ["breadcrumb__step"]] [
-                                      span ~a:[a_class ["breadcrumb__step__title"]] [
-                                          txt (s_ "Input credential");
-                                        ];
-                                      span ~a:[a_class ["breadcrumb__step__short-title"]; a_title (s_ "Input credential")] [
-                                          txt (s_ "Step 1");
-                                        ];
-                                    ];
-                                  div ~a:[a_class ["breadcrumb__step-separator"]] [];
-                                  div ~a:[a_class ["breadcrumb__step"]] [
-                                      span ~a:[a_class ["breadcrumb__step__title"]] [
-                                          txt (s_ "Answer to questions");
-                                        ];
-                                      span ~a:[a_class ["breadcrumb__step__short-title"]; a_title (s_ "Answer to questions")] [
-                                          txt (s_ "Step 2");
-                                        ];
-                                    ];
-                                  div ~a:[a_class ["breadcrumb__step-separator"]] [];
-                                  div ~a:[a_class ["breadcrumb__step"]] [
-                                      span ~a:[a_class ["breadcrumb__step__title"]] [
-                                          txt (s_ "Review and encrypt");
-                                        ];
-                                      span ~a:[a_class ["breadcrumb__step__short-title"]; a_title (s_ "Review and encrypt")] [
-                                          txt (s_ "Step 3");
-                                        ];
-                                    ];
-                                  div ~a:[a_class ["breadcrumb__step-separator"]] [];
-                                  div ~a:[a_class ["breadcrumb__step"]] [
-                                      span ~a:[a_class ["breadcrumb__step__title"]] [
-                                          txt (s_ "Authenticate");
-                                        ];
-                                      span ~a:[a_class ["breadcrumb__step__short-title"]; a_title (s_ "Authenticate")] [
-                                          txt (s_ "Step 4");
-                                        ];
-                                    ];
-                                  div ~a:[a_class ["breadcrumb__step-separator"]] [];
-                                  div ~a:[a_class ["breadcrumb__step breadcrumb__step--current"]] [
-                                      span ~a:[a_class ["breadcrumb__step__title"]] [
-                                          txt (s_ "Confirm");
-                                        ];
-                                      span ~a:[a_class ["breadcrumb__step__short-title"]; a_title (s_ "Confirm")] [
-                                          txt (s_ "Step 5");
-                                        ];
-                                    ];
-                                  div ~a:[a_class ["breadcrumb__step-separator"]] [];
-                                ] in
     let result, step_title =
       match result with
-      | Ok (hash, weight, email) ->
+      | Ok (user, hash, revote, weight, email) ->
+         let this_is_a_revote =
+           if revote then (
+             span [
+                 txt @@ s_ "This is a revote.";
+                 txt " ";
+               ]
+           ) else txt ""
+         in
          let your_weight_is =
            if not Weight.(is_int weight 1) then
              span [
@@ -869,8 +736,11 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
                ]
            else txt ""
          in
-         [txt (s_ " has been accepted.");
+         [txt (s_ " as user ");
+          em [txt user.user_name];
+          txt (s_ " has been accepted.");
           txt " ";
+          this_is_a_revote;
           your_weight_is;
           txt (s_ "Your smart ballot tracker is ");
           b ~a:[a_id "ballot_tracker"] [
@@ -889,7 +759,7 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
          ], s_ "FAIL!"
     in
     let content = [
-        progress_responsive;
+        progress_responsive_step5 l;
         div ~a:[a_class ["current_step"]] [
             txt step_title;
           ];
@@ -905,7 +775,7 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
       ] in
     let title = br_truncate name in
     let full_title = name in
-    responsive_base ~full_title ~title ~content ~uuid ()
+    base ~full_title ~title ~content ~uuid ()
 
   let pretty_ballots election =
     let* l = get_preferred_gettext () in
@@ -913,14 +783,6 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
     let open (val election : Site_common_sig.ELECTION_LWT) in
     let uuid = election.e_uuid in
     let* hashes = Web_persist.get_ballot_hashes uuid in
-    let* result = Web_persist.get_election_result uuid in
-    let result =
-      Option.map (
-          election_result_of_string
-            read_result Yojson.Safe.read_json
-            Yojson.Safe.read_json Yojson.Safe.read_json
-        ) result
-    in
     let* audit_cache = Web_persist.get_audit_cache uuid in
     let show_weights = audit_cache.cache_checksums.ec_weights <> None in
     let title = br_truncate election.e_name ^ " — " ^ s_ "Accepted ballots" in
@@ -944,31 +806,38 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
            [txt (s_ "Go back to election")]
            (uuid, ())]
     in
-    let number = match !nballots, result with
-      | n, None ->
+    let* number =
+      let n = !nballots in
+      let* x = Web_persist.get_sized_encrypted_tally uuid in
+      let x = Option.map (sized_encrypted_tally_of_string read_hash) x in
+      match x with
+      | None ->
          div [
              txt (string_of_int n);
              txt (s_ " ballot(s) have been accepted so far.");
            ]
-      | n, Some r when Weight.(is_int r.num_tallied n) ->
+         |> Lwt.return
+      | Some x when x.sized_num_tallied = n ->
          div [
              txt (string_of_int n);
              txt (s_ " ballot(s) have been accepted.");
            ]
-      | n, Some r -> (* should not happen *)
+         |> Lwt.return
+      | Some x -> (* should not happen *)
          div [
              txt (string_of_int n);
              txt (s_ " ballot(s) have been accepted, and ");
-             txt (Weight.to_string r.num_tallied);
+             txt (string_of_int x.sized_num_tallied);
              txt (s_ " have been tallied.");
            ]
+         |> Lwt.return
     in
     let content = [
         number;
         ul ballots;
         links;
       ] in
-    responsive_base ~title ~content ~uuid ()
+    base ~title ~content ~uuid ()
 
   let booth () =
     let* l = get_preferred_gettext () in
@@ -1087,8 +956,12 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
       div ~a:[a_id "booth_div"; a_style "display:none;"] [
           div ~a:[a_id "header"] [
               div ~a:[a_style "float: left; padding: 15px;"] [
+                  let src =
+                    Eliom_uri.make_string_uri ~service:logo ()
+                    |> Eliom_content.Xml.uri_of_string
+                  in
                   img ~alt:(s_ "Election server") ~a:[a_height 70]
-                    ~src:(static "logo.png") ();
+                    ~src ();
                 ];
               div ~a:[a_style "float: right; padding: 15px;"] [
                   img ~alt:"" ~a:[a_height 70]
@@ -1129,7 +1002,7 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
     let open (val l) in
     let title = s_ "Condorcet-Schulze method" in
     let content = schulze_content l q r in
-    responsive_base ~title ~content ()
+    base ~title ~content ()
 
   let majority_judgment_select uuid question =
     let* l = get_preferred_gettext () in
@@ -1138,7 +1011,7 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
     let form =
       get_form ~service:method_mj
         (fun (uuidn, (questionn, ngradesn)) -> [
-             input ~input_type:`Hidden ~name:uuidn ~value:uuid (user raw_string_of_uuid);
+             input ~input_type:`Hidden ~name:uuidn ~value:uuid (user Uuid.unwrap);
              input ~input_type:`Hidden ~name:questionn ~value:question int;
              txt (s_ "Number of grades:");
              txt " ";
@@ -1179,7 +1052,7 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
         form;
       ]
     in
-    responsive_base ~title ~content ()
+    base ~title ~content ()
 
   let majority_judgment q r =
     let* l = get_preferred_gettext () in
@@ -1195,7 +1068,7 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
     let form =
       get_form ~service:method_stv
         (fun (uuidn, (questionn, nseatsn)) -> [
-             input ~input_type:`Hidden ~name:uuidn ~value:uuid (user raw_string_of_uuid);
+             input ~input_type:`Hidden ~name:uuidn ~value:uuid (user Uuid.unwrap);
              input ~input_type:`Hidden ~name:questionn ~value:question int;
              txt (s_ "Number of seats:");
              txt " ";
@@ -1297,6 +1170,6 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
         div [invalid];
       ]
     in
-    responsive_base ~title ~content ()
+    base ~title ~content ()
 
 end
