@@ -25,11 +25,9 @@ open Belenios_platform
 open Platform
 open Belenios_core
 open Belenios
-open Serializable_builtin_t
 open Serializable_j
 open Signatures
 open Common
-open Web_serializable_builtin_t
 open Web_serializable_j
 open Web_common
 
@@ -70,7 +68,7 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
     let tallied = filter `Tallied in
     let archived = filter `Archived in
     let sort l =
-      List.sort (fun (_, x, _) (_, y, _) -> datetime_compare x y) l |>
+      List.sort (fun (_, x, _) (_, y, _) -> Datetime.compare x y) l |>
         List.map (fun (x, _, y) -> x, y)
     in
     return (sort draft, sort elections, sort tallied, sort archived)
@@ -92,9 +90,9 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
   let with_metadata_check_owner uuid f =
     let* user = Eliom_reference.get Web_state.site_user in
     let* metadata = Web_persist.get_election_metadata uuid in
-    match user, metadata.e_owner with
-    | Some x, Some y when Accounts.check x y -> f metadata
-    | _, _ -> forbidden ()
+    match user with
+    | Some (_, a, _) when Accounts.check a metadata.e_owners -> f metadata
+    | _ -> forbidden ()
 
   let without_site_user ?fallback () f =
     let* l = get_preferred_gettext () in
@@ -121,6 +119,18 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
         return cont
       )
 
+  let () =
+    Any.register ~service:sealing
+      (fun () () ->
+        let* site_user = Eliom_reference.get Web_state.site_user in
+        match site_user with
+        | None -> forbidden ()
+        | Some _ ->
+           match !Web_config.sealing with
+           | Some (file, content_type) -> File.send ~content_type file
+           | None -> fail_http `Not_found
+      )
+
   let () = Html.register ~service:admin
              (fun () () ->
                let* site_user = Eliom_reference.get Web_state.site_user in
@@ -135,7 +145,7 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
                        if b then (
                          return_true
                        ) else (
-                         let account_consent = Some (now ()) in
+                         let account_consent = Some (Datetime.now ()) in
                          let a = {a with account_consent} in
                          let* () = Accounts.update_account {a with account_consent} in
                          return_false
@@ -294,11 +304,11 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
       )
 
   let with_draft_election_ro uuid f =
-    let@ u = with_site_user in
+    let@ _, a, _ = with_site_user in
     let* election = Web_persist.get_draft_election uuid in
     match election with
     | None -> fail_http `Not_found
-    | Some se -> if Accounts.check u se.se_owner then f se else forbidden ()
+    | Some se -> if Accounts.check a se.se_owners then f se else forbidden ()
 
   let () =
     Any.register ~service:election_draft
@@ -312,9 +322,9 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
     Any.register ~service:election_draft_trustees
       (fun uuid () ->
         let@ se = with_draft_election_ro uuid in
-        match se.se_threshold_trustees with
-        | None -> Pages_admin.election_draft_trustees uuid se () >>= Html.send
-        | Some _ -> redir_preapply election_draft_threshold_trustees uuid ()
+        match se.se_trustees with
+        | `Basic _ -> Pages_admin.election_draft_trustees uuid se () >>= Html.send
+        | `Threshold _ -> redir_preapply election_draft_threshold_trustees uuid ()
       )
 
   let () =
@@ -334,7 +344,7 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
       )
 
   let with_draft_election ?(save = true) uuid f =
-    let@ u = with_site_user in
+    let@ _, a, _ = with_site_user in
     let* l = get_preferred_gettext () in
     let open (val l) in
     let@ () = Web_election_mutex.with_lock uuid in
@@ -342,7 +352,7 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
     match election with
     | None -> fail_http `Not_found
     | Some se ->
-       if Accounts.check u se.se_owner then (
+       if Accounts.check a se.se_owners then (
          Lwt.catch
            (fun () ->
              let* r = f se in
@@ -559,11 +569,11 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
     Any.register ~service:election_draft_preview
       (fun (uuid, ()) () ->
         let@ se = with_draft_election_ro uuid in
-        let version = Option.value se.se_version ~default:0 in
+        let version = se.se_version in
         let group = se.se_group in
         let module G = (val Group.of_string ~version group : GROUP) in
         let params = {
-            e_version = Option.value se.se_version ~default:0;
+            e_version = version;
             e_description = se.se_questions.t_description;
             e_name = se.se_questions.t_name;
             e_questions = se.se_questions.t_questions;
@@ -699,9 +709,9 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
       )
 
   let ensure_trustees_mode uuid se mode =
-    match se.se_threshold_trustees, mode with
-    | None, `Basic | Some _, `Threshold _ -> Lwt.return se
-    | Some _, `Basic | None, `Threshold _ ->
+    match se.se_trustees, mode with
+    | `Basic _, `Basic | `Threshold _, `Threshold _ -> Lwt.return se
+    | `Threshold _, `Basic | `Basic _, `Threshold _ ->
        let* () = Api_drafts.put_draft_trustees_mode uuid se mode in
        let* x = Web_persist.get_draft_election uuid in
        match x with
@@ -715,7 +725,15 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
     if is_email trustee_address then (
       let* se = ensure_trustees_mode uuid se mode in
       let open Belenios_api.Serializable_t in
-      let trustee = {trustee_address; trustee_name; trustee_token = None; trustee_state = None} in
+      let trustee =
+        {
+          trustee_address = Some trustee_address;
+          trustee_name;
+          trustee_token = None;
+          trustee_state = None;
+          trustee_key = None
+        }
+      in
       let* () = Api_drafts.post_draft_trustees uuid se trustee in
       redir_preapply election_draft_trustees uuid ()
     ) else (
@@ -750,7 +768,7 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
              >>= Html.send
            ) else (
              Printf.sprintf "%s/draft/credentials.html#%s-%s"
-               !Web_config.prefix (raw_string_of_uuid uuid) token
+               !Web_config.prefix (Uuid.unwrap uuid) token
              |> String_redirection.send
            )
       )
@@ -766,8 +784,7 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
     | Some se ->
        if se.se_public_creds <> token then forbidden () else
          if se.se_public_creds_received then forbidden () else
-           let* creds = Lwt_stream.to_string creds in
-           let creds = split_lines creds in
+           let creds = public_credentials_of_string creds in
            let* () = Api_drafts.submit_public_credentials uuid se creds in
            Pages_admin.election_draft_credentials_done se () >>= Html.send
 
@@ -775,16 +792,17 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
     Any.register ~service:election_draft_credentials_post
       (fun (uuid, token) creds ->
         let@ () = without_site_user () in
-        let s = Lwt_stream.of_string creds in
-        wrap_handler (fun () -> handle_credentials_post uuid token s)
+        wrap_handler (fun () -> handle_credentials_post uuid token creds)
       )
 
   let () =
     Any.register ~service:election_draft_credentials_post_file
       (fun (uuid, token) creds ->
         let@ () = without_site_user () in
-        let s = Lwt_io.chars_of_file creds.Ocsigen_extensions.tmp_filename in
-        wrap_handler (fun () -> handle_credentials_post uuid token s)
+        let fname = creds.Ocsigen_extensions.tmp_filename in
+        let* creds = Lwt_stream.to_string (Lwt_io.chars_of_file fname) in
+        let* () = Lwt_unix.unlink fname in
+        wrap_handler (fun () -> handle_credentials_post uuid token creds)
       )
 
   let () =
@@ -830,12 +848,12 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
         let open (val l) in
         let@ () =
           without_site_user
-            ~fallback:(fun u ->
+            ~fallback:(fun (_, a, _) ->
               let* election = Web_persist.get_draft_election uuid in
               match election with
               | None -> fail_http `Not_found
               | Some se ->
-                 if Accounts.check u se.se_owner then (
+                 if Accounts.check a se.se_owners then (
                    Pages_admin.election_draft_trustees ~token uuid se () >>= Html.send
                  ) else forbidden ()
             ) ()
@@ -844,7 +862,12 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
         match election with
         | None -> fail_http `Not_found
         | Some se ->
-           match List.find_opt (fun t -> t.st_token = token) se.se_public_keys with
+           let ts =
+             match se.se_trustees with
+             | `Basic x -> x.dbp_trustees
+             | `Threshold _ -> []
+           in
+           match List.find_opt (fun t -> t.st_token = token) ts with
            | None -> forbidden ()
            | Some t ->
               if t.st_public_key <> "" then
@@ -853,7 +876,7 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
                 Pages_common.generic_page ~title msg () >>= Html.send ~code:403
               else (
                 Printf.sprintf "%s/draft/trustee.html#%s-%s"
-                  !Web_config.prefix (raw_string_of_uuid uuid) token
+                  !Web_config.prefix (Uuid.unwrap uuid) token
                 |> String_redirection.send
               )
       )
@@ -878,15 +901,20 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
                 match election with
                 | None -> fail_http `Not_found
                 | Some se ->
-                   let&* t = List.find_opt (fun x -> token = x.st_token) se.se_public_keys in
+                   let ts =
+                     match se.se_trustees with
+                     | `Basic x -> x.dbp_trustees
+                     | `Threshold _ -> []
+                   in
+                   let&* t = List.find_opt (fun x -> token = x.st_token) ts in
                    if t.st_public_key <> "" then
                      let msg = s_ "A public key already existed, the key you've just uploaded has been ignored!" in
                      let title = s_ "Error" in
                      return_some (title, msg, 400)
                    else
-                     let version = Option.value se.se_version ~default:0 in
+                     let version = se.se_version in
                      let module G = (val Group.of_string ~version se.se_group : GROUP) in
-                     let module Trustees = (val Trustees.get_by_version (Option.value se.se_version ~default:0)) in
+                     let module Trustees = (val Trustees.get_by_version version) in
                      let pk = trustee_public_key_of_string G.read public_key in
                      let module K = Trustees.MakeCombinator (G) in
                      if not (K.check [`Single pk]) then
@@ -922,8 +950,10 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
         let@ se = with_draft_election ~save:false uuid in
         Lwt.catch
           (fun () ->
-            let* () = Api_drafts.validate_election account uuid se in
-            redir_preapply election_admin uuid ()
+            if Accounts.check account se.se_owners then (
+              let* () = Api_drafts.validate_election uuid se in
+              redir_preapply election_admin uuid ()
+            ) else Lwt.fail (Failure "Forbidden")
           )
           (fun e ->
             Pages_admin.new_election_failure (`Exception e) () >>= Html.send
@@ -951,7 +981,7 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
   let () =
     Any.register ~service:election_draft_import_post
       (fun uuid from_s ->
-        let from = uuid_of_raw_string from_s in
+        let from = Uuid.wrap from_s in
         let@ se = with_draft_election ~save:false uuid in
         let@ _ = with_metadata_check_owner from in
         let* l = get_preferred_gettext () in
@@ -997,7 +1027,7 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
   let () =
     Any.register ~service:election_draft_import_trustees_post
       (fun uuid from ->
-        let from = uuid_of_raw_string from in
+        let from = Uuid.wrap from in
         let@ se = with_draft_election ~save:false uuid in
         let@ _ = with_metadata_check_owner from in
         let* metadata = Web_persist.get_election_metadata from in
@@ -1029,7 +1059,7 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
     let* metadata = Web_persist.get_election_metadata uuid in
     let* site_user = Eliom_reference.get Web_state.site_user in
     match site_user with
-    | Some x when (match metadata.e_owner with None -> false | Some o -> Accounts.check x o) ->
+    | Some (_, a, _) when Accounts.check a metadata.e_owners ->
        let* status = Api_elections.get_election_status uuid in
        let module W = (val election) in
        let* pending_server_shuffle =
@@ -1049,14 +1079,12 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
            let cc = nh_ciphertexts_of_string W.G.read cc in
            let* shuffle = W.E.shuffle_ciphertexts cc in
            let shuffle = string_of_shuffle W.G.write shuffle in
-           let* x = Web_persist.append_to_shuffles election shuffle in
+           let* x = Web_persist.append_to_shuffles election 1 shuffle in
            match x with
-           | Some h ->
-              let sh = {sh_trustee = "server"; sh_hash = h; sh_name = Some "server"} in
-              let* () = Web_persist.add_shuffle_hash uuid sh in
+           | Some _ ->
               Web_persist.remove_audit_cache uuid
            | None ->
-              Lwt.fail (Failure (Printf.sprintf (f_ "Automatic shuffle by server has failed for election %s!") (raw_string_of_uuid uuid)))
+              Lwt.fail (Failure (Printf.sprintf (f_ "Automatic shuffle by server has failed for election %s!") (Uuid.unwrap uuid)))
          ) else return_unit
        in
        Pages_admin.election_admin ?shuffle_token ?tally_token election metadata status () >>= Html.send
@@ -1095,14 +1123,14 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
 
   let parse_datetime_from_post l x =
     let open (val l : Belenios_ui.I18n.GETTEXT) in
-    try raw_datetime_of_string x with
+    try Datetime.wrap x with
     | _ -> Printf.ksprintf failwith (f_ "%s is not a valid date!") x
 
   let () =
     Any.register ~service:election_hide_result
       (fun uuid date ->
         let@ date = fun cont ->
-          match Option.wrap raw_datetime_of_string date with
+          match Option.wrap Datetime.wrap date with
           | None ->
              let* l = get_preferred_gettext () in
              let open (val l) in
@@ -1110,7 +1138,7 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
              let msg = Printf.sprintf (f_ "%s is not a valid date!") date in
              Pages_common.generic_page ~title:(s_ "Error") ~service msg () >>= Html.send
           | Some t ->
-             cont @@ unixfloat_of_datetime t
+             cont @@ Datetime.to_unixfloat t
         in
         election_set_result_hidden uuid (Some date)
       )
@@ -1139,8 +1167,8 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
            let open Belenios_api.Serializable_t in
            let dates =
              {
-               auto_date_open = Option.map unixfloat_of_datetime e_auto_open;
-               auto_date_close = Option.map unixfloat_of_datetime e_auto_close;
+               auto_date_open = Option.map Datetime.to_unixfloat e_auto_open;
+               auto_date_close = Option.map Datetime.to_unixfloat e_auto_close;
              }
            in
            let* () = Api_elections.set_election_auto_dates uuid dates in
@@ -1223,7 +1251,7 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
               let* metadata = Web_persist.get_election_metadata uuid in
               let* site_user = Eliom_reference.get Web_state.site_user in
               match site_user with
-              | Some x when (match metadata.e_owner with None -> false | Some o -> Accounts.check x o) -> return_true
+              | Some (_, a, _) when Accounts.check a metadata.e_owners -> return_true
               | _ -> return_false
             ) else return_true
           in
@@ -1232,12 +1260,7 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
             match result with
             | None -> fail_http `Not_found
             | Some result ->
-               let result =
-                 election_result_of_string
-                   Yojson.Safe.read_json Yojson.Safe.read_json
-                   Yojson.Safe.read_json Yojson.Safe.read_json
-                   result
-               in
+               let result = election_result_of_string Yojson.Safe.read_json result in
                match result.result with
                | `List xs ->
                   (match List.nth_opt xs index with
@@ -1260,7 +1283,7 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
     if b then copy_file src dst else return_unit
 
   let make_archive uuid =
-    let uuid_s = raw_string_of_uuid uuid in
+    let uuid_s = Uuid.unwrap uuid in
     let* temp_dir =
       Lwt_preemptive.detach (fun () ->
           let temp_dir = Filename.temp_file "belenios" "archive" in
@@ -1275,11 +1298,7 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
       Lwt_list.iter_p (fun x ->
           try_copy_file (uuid /// x) (temp_dir // "public" // x)
         ) [
-          "election.json";
-          "trustees.json";
-          "public_creds.txt";
-          "ballots.jsons";
-          "result.json";
+          Uuid.unwrap uuid ^ ".bel";
         ]
     in
     let* () =
@@ -1351,18 +1370,18 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
         let open (val l) in
         let* state = Web_persist.get_election_state uuid in
         match state with
-        | `EncryptedTally _ ->
+        | `EncryptedTally ->
            let* x = find_trustee_id uuid token in
            (match x with
             | Some trustee_id ->
                let* pds = Web_persist.get_partial_decryptions uuid in
-               if List.mem_assoc trustee_id pds then (
+               if List.exists (fun x -> x.owned_owner = trustee_id) pds then (
                  Pages_common.generic_page ~title:(s_ "Error")
                    (s_ "Your partial decryption has already been received and checked!")
                    () >>= Html.send
                ) else (
                  Printf.sprintf "%s/election/trustees.html#%s-%s"
-                   !Web_config.prefix (raw_string_of_uuid uuid) token
+                   !Web_config.prefix (Uuid.unwrap uuid) token
                  |> String_redirection.send
                )
             | None -> forbidden ()
@@ -1396,7 +1415,7 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
         let* () =
           let* state = Web_persist.get_election_state uuid in
           match state with
-          | `EncryptedTally _ -> return ()
+          | `EncryptedTally -> return ()
           | _ -> Lwt.fail TallyEarlyError
         in
         let* trustee_id =
@@ -1407,7 +1426,9 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
         in
         let* pds = Web_persist.get_partial_decryptions uuid in
         let* () =
-          if List.mem_assoc trustee_id pds then Lwt.fail TallyEarlyError else return ()
+          if List.exists (fun x -> x.owned_owner = trustee_id) pds then
+            Lwt.fail TallyEarlyError
+          else return ()
         in
         let* () =
           if trustee_id > 0 then return () else fail_http `Not_found
@@ -1429,9 +1450,12 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
         in
         let pk = pks.(trustee_id-1).trustee_public_key in
         let pd = partial_decryption_of_string W.G.read partial_decryption in
-        let et = uuid /// string_of_election_file ESETally in
-        let* et = Lwt_io.chars_of_file et |> Lwt_stream.to_string in
-        let et = encrypted_tally_of_string W.G.read et in
+        let* et =
+          let* x = Web_persist.get_latest_encrypted_tally election in
+          match x with
+          | None -> assert false
+          | Some x -> Lwt.return @@ encrypted_tally_of_string W.G.read x
+        in
         if string_of_partial_decryption W.G.write pd = partial_decryption && W.E.check_factor et pk pd then (
           let pd = trustee_id, partial_decryption in
           let* () = Web_persist.add_partial_decryption uuid pd in
@@ -1446,47 +1470,34 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
       ))
 
   let handle_election_tally_release uuid () =
-    let@ () = render_tally_early_error_as_forbidden in
     let@ _ = with_metadata_check_owner uuid in
     let* l = get_preferred_gettext () in
     let open (val l) in
-    let@ election = with_election uuid in
-    let* x = Api_elections.release_tally election in
-    match x with
-    | Ok () -> redir_preapply election_home (uuid, ()) ()
-    | Error `Forbidden -> forbidden ()
-    | Error (`CombinationError e) ->
-       let msg =
-         Printf.sprintf
-           (f_ "An error occurred while computing the result (%s). Most likely, it means that some trustee has not done his/her job.")
-           (Trustees.string_of_combination_error e)
-       in
-       Pages_common.generic_page ~title:(s_ "Error") msg () >>= Html.send
+    Lwt.catch
+      (fun () ->
+        let* () = Web_persist.release_tally uuid in
+        redir_preapply election_home (uuid, ()) ()
+      )
+      (fun e ->
+        let msg =
+          Printf.sprintf
+            (f_ "An error occurred while computing the result (%s). Most likely, it means that some trustee has not done his/her job.")
+            (Printexc.to_string e)
+        in
+        Pages_common.generic_page ~title:(s_ "Error") msg () >>= Html.send
+      )
 
   let () =
     Any.register ~service:election_tally_release
       handle_election_tally_release
 
-  let handle_api_elections_return uuid metadata = function
-    | false -> forbidden ()
-    | true ->
-       let* state = Web_persist.get_election_state uuid in
-       match state with
-       | `EncryptedTally _ ->
-          let trustees = Option.value metadata.e_trustees ~default:[] in
-          if List.exists (fun x -> x <> "server") trustees then
-            redir_preapply election_admin uuid ()
-          else
-            handle_election_tally_release uuid ()
-       | _ -> redir_preapply election_admin uuid ()
-
   let () =
     Any.register ~service:election_compute_encrypted_tally
       (fun uuid () ->
-        let@ metadata = with_metadata_check_owner uuid in
+        let@ _ = with_metadata_check_owner uuid in
         let@ election = with_election uuid in
-        let* x = Api_elections.compute_encrypted_tally election metadata in
-        handle_api_elections_return uuid metadata x
+        let* _ = Api_elections.compute_encrypted_tally election in
+        redir_preapply election_admin uuid ()
       )
 
   let () =
@@ -1502,7 +1513,7 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
         match expected_token with
         | Some x when token = x.tk_token ->
            Printf.sprintf "%s/election/shuffle.html#%s-%s"
-             !Web_config.prefix (raw_string_of_uuid uuid) token
+             !Web_config.prefix (Uuid.unwrap uuid) token
            |> String_redirection.send
         | _ -> forbidden ()
       )
@@ -1523,12 +1534,10 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
         | Some x when token = x.tk_token ->
            Lwt.catch
              (fun () ->
-               let* y = Web_persist.append_to_shuffles election shuffle in
+               let* y = Web_persist.append_to_shuffles election x.tk_trustee_id shuffle in
                match y with
-               | Some h ->
+               | Some _ ->
                   let* () = Web_persist.clear_shuffle_token uuid in
-                  let sh = {sh_trustee = x.tk_trustee; sh_hash = h; sh_name = x.tk_name} in
-                  let* () = Web_persist.add_shuffle_hash uuid sh in
                   let* () = Web_persist.remove_audit_cache uuid in
                   Pages_common.generic_page ~title:(s_ "Success") (s_ "The shuffle has been successfully applied!") () >>= Html.send
                | None ->
@@ -1558,17 +1567,17 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
   let () =
     Any.register ~service:election_shuffler_skip
       (fun () (uuid, trustee) ->
-        let@ metadata = with_metadata_check_owner uuid in
-        let* () = Api_elections.skip_shuffler uuid metadata trustee in
+        let@ _ = with_metadata_check_owner uuid in
+        let* () = Api_elections.skip_shuffler uuid trustee in
         redir_preapply election_admin uuid ()
       )
 
   let () =
     Any.register ~service:election_decrypt (fun uuid () ->
-        let@ metadata = with_metadata_check_owner uuid in
+        let@ _ = with_metadata_check_owner uuid in
         let@ election = with_election uuid in
-        let* x = Api_elections.finish_shuffling election metadata in
-        handle_api_elections_return uuid metadata x
+        let* _ = Api_elections.finish_shuffling election in
+        redir_preapply election_admin uuid ()
       )
 
   let () =
@@ -1603,12 +1612,12 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
       (fun (uuid, token) () ->
         let@ () =
           without_site_user
-            ~fallback:(fun u ->
+            ~fallback:(fun (_, a, _) ->
               let* election = Web_persist.get_draft_election uuid in
               match election with
               | None -> fail_http `Not_found
               | Some se ->
-                 if Accounts.check u se.se_owner then (
+                 if Accounts.check a se.se_owners then (
                    Pages_admin.election_draft_threshold_trustees ~token uuid se () >>= Html.send
                  ) else forbidden ()
             ) ()
@@ -1618,7 +1627,7 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
         | None -> fail_http `Not_found
         | Some _ ->
            Printf.sprintf "%s/draft/threshold-trustee.html#%s-%s"
-             !Web_config.prefix (raw_string_of_uuid uuid) token
+             !Web_config.prefix (Uuid.unwrap uuid) token
            |> String_redirection.send
       )
 
@@ -1640,11 +1649,12 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
               match election with
               | None -> fail_http `Not_found
               | Some se ->
-                 let ts =
-                   match se.se_threshold_trustees with
-                   | None -> failwith "No threshold trustees"
-                   | Some xs -> Array.of_list xs
+                 let dtp =
+                   match se.se_trustees with
+                   | `Basic _ -> failwith "No threshold trustees"
+                   | `Threshold x -> x
                  in
+                 let ts = Array.of_list dtp.dtp_trustees in
                  let i, t =
                    match Array.findi (fun i x ->
                              if token = x.stt_token then Some (i, x) else None
@@ -1667,9 +1677,9 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
                        | Some y -> y
                      ) ts
                  in
-                 let version = Option.value se.se_version ~default:0 in
+                 let version = se.se_version in
                  let module G = (val Group.of_string ~version se.se_group : GROUP) in
-                 let module Trustees = (val Trustees.get_by_version (Option.value se.se_version ~default:0)) in
+                 let module Trustees = (val Trustees.get_by_version version) in
                  let module P = Trustees.MakePKI (G) (LwtRandom) in
                  let module C = Trustees.MakeChannels (G) (LwtRandom) (P) in
                  let module K = Trustees.MakePedersen (G) (LwtRandom) (P) (C) in
@@ -1713,7 +1723,7 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
                         K.step2 (get_certs ());
                         Array.iter (fun x -> x.stt_step <- Some 3) ts;
                       with e ->
-                        se.se_threshold_error <- Some (Printexc.to_string e)
+                        dtp.dtp_error <- Some (Printexc.to_string e)
                      ); return_unit
                    ) else return_unit
                  in
@@ -1728,7 +1738,7 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
                         done;
                         Array.iter (fun x -> x.stt_step <- Some 5) ts
                       with e ->
-                        se.se_threshold_error <- Some (Printexc.to_string e)
+                        dtp.dtp_error <- Some (Printexc.to_string e)
                      ); return_unit
                    ) else return_unit
                  in
@@ -1743,10 +1753,10 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
                                            | Some y -> voutput_of_string G.read y
                                          ) ts in
                         let p = K.step6 certs polynomials voutputs in
-                        se.se_threshold_parameters <- Some (string_of_threshold_parameters G.write p);
+                        dtp.dtp_parameters <- Some (string_of_threshold_parameters G.write p);
                         Array.iter (fun x -> x.stt_step <- Some 7) ts
                       with e ->
-                        se.se_threshold_error <- Some (Printexc.to_string e)
+                        dtp.dtp_error <- Some (Printexc.to_string e)
                      ); return_unit
                    ) else return_unit
                  in
@@ -1999,31 +2009,31 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
       )
 
   let extract_automatic_data_draft uuid_s =
-    let uuid = uuid_of_raw_string uuid_s in
+    let uuid = Uuid.wrap uuid_s in
     let* se = Web_persist.get_draft_election uuid in
     let&* se in
     let t = Option.value se.se_creation_date ~default:default_creation_date in
-    let next_t = datetime_add t (day days_to_delete) in
+    let next_t = Period.add t (Period.day days_to_delete) in
     return_some (`Destroy, uuid, next_t)
 
   let extract_automatic_data_validated uuid_s =
-    let uuid = uuid_of_raw_string uuid_s in
+    let uuid = Uuid.wrap uuid_s in
     let* election = Web_persist.get_raw_election uuid in
     let&* _ = election in
     let* state = Web_persist.get_election_state uuid in
     let* dates = Web_persist.get_election_dates uuid in
     match state with
-    | `Open | `Closed | `Shuffling | `EncryptedTally _ ->
+    | `Open | `Closed | `Shuffling | `EncryptedTally ->
        let t = Option.value dates.e_finalization ~default:default_validation_date in
-       let next_t = datetime_add t (day days_to_delete) in
+       let next_t = Period.add t (Period.day days_to_delete) in
        return_some (`Delete, uuid, next_t)
     | `Tallied ->
        let t = Option.value dates.e_tally ~default:default_tally_date in
-       let next_t = datetime_add t (day days_to_archive) in
+       let next_t = Period.add t (Period.day days_to_archive) in
        return_some (`Archive, uuid, next_t)
     | `Archived ->
        let t = Option.value dates.e_archive ~default:default_archive_date in
-       let next_t = datetime_add t (day days_to_delete) in
+       let next_t = Period.add t (Period.day days_to_delete) in
        return_some (`Delete, uuid, next_t)
 
   let try_extract extract x =
@@ -2046,14 +2056,14 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
         )
 
   let process_election_for_data_policy (action, uuid, next_t) =
-    let uuid_s = raw_string_of_uuid uuid in
-    let now = now () in
+    let uuid_s = Uuid.unwrap uuid in
+    let now = Datetime.now () in
     let action, comment = match action with
       | `Destroy -> Api_drafts.delete_draft, "destroyed"
       | `Delete -> delete_election, "deleted"
       | `Archive -> Api_elections.archive_election, "archived"
     in
-    if datetime_compare now next_t > 0 then (
+    if Datetime.compare now next_t > 0 then (
       let* () = action uuid in
       return (
           Printf.ksprintf Ocsigen_messages.warning
