@@ -23,10 +23,8 @@ open Lwt
 open Lwt.Syntax
 open Belenios_core
 open Belenios
-open Serializable_builtin_t
 open Serializable_j
 open Common
-open Web_serializable_builtin_t
 open Web_serializable_j
 open Web_common
 
@@ -92,10 +90,9 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Site_admin : Si
   let () =
     Any.register ~service:election_submit_ballot_file
       (fun () ballot ->
-        let* ballot =
-          let fname = ballot.Ocsigen_extensions.tmp_filename in
-          Lwt_stream.to_string (Lwt_io.chars_of_file fname)
-        in
+        let fname = ballot.Ocsigen_extensions.tmp_filename in
+        let* ballot = Lwt_stream.to_string (Lwt_io.chars_of_file fname) in
+        let* () = Lwt_unix.unlink fname in
         submit_ballot ballot
       )
 
@@ -128,7 +125,7 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Site_admin : Si
       | None ->
          let msg =
            Printf.sprintf "send_confirmation_email: %s not found"
-             (raw_string_of_uuid uuid)
+             (Uuid.unwrap uuid)
          in
          Lwt.fail (Failure msg)
     in
@@ -152,55 +149,6 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Site_admin : Si
         Lwt.return true)
       (fun _ -> Lwt.return false)
 
-  let cast_ballot election ~rawballot ~user =
-    let module W = (val election : Site_common_sig.ELECTION_LWT) in
-    let uuid = W.election.e_uuid in
-    let* voters = read_file ~uuid "voters.txt" in
-    let voters = match voters with Some xs -> xs | None -> [] in
-    let* email, login, weight =
-      let rec loop = function
-        | x :: xs ->
-           let email, login, weight = split_identity x in
-           if PString.lowercase_ascii login = PString.lowercase_ascii user.user_name then return (email, login, weight) else loop xs
-        | [] -> fail UnauthorizedVoter
-      in loop voters
-    in
-    let show_weight =
-      List.exists
-        (fun x ->
-          let _, _, weight = split_identity_opt x in
-          weight <> None
-        ) voters
-    in
-    let oweight = if show_weight then Some weight else None in
-    let user = string_of_user user in
-    let* state = Web_persist.get_election_state uuid in
-    let voting_open = state = `Open in
-    let* () = if not voting_open then fail ElectionClosed else return_unit in
-    let* r = Web_persist.cast_ballot election ~rawballot ~user ~weight (now ()) in
-    match r with
-    | Ok (hash, revote) ->
-       let* success = send_confirmation_email uuid revote login email oweight hash in
-       let () =
-         if revote then
-           Printf.ksprintf Ocsigen_messages.accesslog
-             "Someone revoted in election %s" (raw_string_of_uuid uuid)
-       in
-       return (hash, weight, success)
-    | Error e ->
-       fail (CastError e)
-
-  let () =
-    Any.register ~service:election_cast_fallback
-      (fun uuid () ->
-        let@ election = with_election uuid in
-        let* ballot = Eliom_reference.get Web_state.ballot in
-        (match ballot with
-         | Some b -> Pages_voter.cast_confirmation election (sha256_b64 b) () >>= Html.send
-         | None -> Pages_voter.lost_ballot election () >>= Html.send
-        )
-      )
-
   let () =
     Any.register ~service:election_cast_confirm
       (fun uuid () ->
@@ -219,7 +167,7 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Site_admin : Si
               let* result =
                 Lwt.catch
                   (fun () ->
-                    let* hash = cast_ballot election ~rawballot ~user in
+                    let* hash = Api_elections.cast_ballot send_confirmation_email election ~rawballot ~user in
                     return (Ok hash)
                   )
                   (function
@@ -262,14 +210,9 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Site_admin : Si
              let* result = Web_persist.get_election_result uuid in
              match result with
              | Some result ->
-                let result =
-                  election_result_of_string
-                    read_result Yojson.Safe.read_json
-                    Yojson.Safe.read_json Yojson.Safe.read_json
-                    result
-                in
+                let result = election_result_of_string read_result result in
                 (match (result.result :> raw_result).(question) with
-                 | RNonHomomorphic ballots -> continuation ballots
+                 | `NonHomomorphic ballots -> continuation ballots
                  | _ -> failwith "handle_method"
                 )
              | None ->
@@ -365,14 +308,14 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Site_admin : Si
 
   let content_type_of_file = function
     | ESRaw -> "application/json; charset=utf-8"
-    | ESTrustees | ESETally | ESResult -> "application/json"
-    | ESBallots | ESShuffles -> "text/plain" (* should be "application/json-seq", but we don't use RS *)
-    | ESCreds | ESRecords | ESVoters -> "text/plain"
+    | ESETally | ESResult -> "application/json"
+    | ESArchive _ -> "application/x-belenios"
+    | ESRecords | ESVoters -> "text/plain"
 
   let handle_pseudo_file uuid f site_user =
     let* confidential =
       match f with
-      | ESRaw | ESTrustees | ESBallots | ESETally | ESCreds | ESShuffles -> return false
+      | ESRaw | ESETally | ESArchive _ -> return false
       | ESRecords | ESVoters -> return true
       | ESResult ->
          let* hidden = Web_persist.get_election_result_hidden uuid in
@@ -384,13 +327,43 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Site_admin : Si
       if confidential then (
         let* metadata = Web_persist.get_election_metadata uuid in
         match site_user with
-        | Some x when (match metadata.e_owner with None -> false | Some o -> Accounts.check x o) -> return_true
+        | Some (_, a, _) when Accounts.check a metadata.e_owners -> return_true
         | _ -> return_false
       ) else return_true
     in
     if allowed then (
       let content_type = content_type_of_file f in
-      File.send ~content_type (uuid /// string_of_election_file f)
+      match f with
+      | ESRaw ->
+         begin
+           let* x = Web_persist.get_raw_election uuid in
+           match x with
+           | Some x ->
+              let* x = String.send (x, content_type) in
+              return @@ cast_unknown_content_kind x
+           | None -> fail_http `Not_found
+         end
+      | ESETally ->
+         begin
+           let@ election = with_election uuid in
+           let* x = Web_persist.get_latest_encrypted_tally election in
+           match x with
+           | Some x ->
+              let* x = String.send (x, content_type) in
+              return @@ cast_unknown_content_kind x
+           | None -> fail_http `Not_found
+         end
+      | ESResult ->
+         begin
+           let* x = Web_persist.get_election_result uuid in
+           match x with
+           | Some x ->
+              let* x = String.send (x, content_type) in
+              return @@ cast_unknown_content_kind x
+           | None -> fail_http `Not_found
+         end
+      | _ ->
+         File.send ~content_type (uuid /// string_of_election_file f)
     ) else forbidden ()
 
   let () =
