@@ -22,9 +22,7 @@
 open Lwt
 open Lwt.Syntax
 open Eliom_service
-open Belenios_core.Serializable_builtin_t
 open Belenios_core.Common
-open Web_serializable_builtin_t
 open Web_serializable_j
 open Web_common
 open Web_auth_sig
@@ -47,7 +45,7 @@ module Make (Web_state : Web_state_sig.S) (Web_services : Web_services_sig.S) (P
   let get_cont login_or_logout x =
     let open Eliom_registration in
     let redir = match x with
-      | `Election uuid -> Redirection (preapply ~service:election_cast_fallback uuid)
+      | `Election uuid -> Redirection (preapply ~service:election_cast_confirm uuid)
       | `Site ContSiteHome -> Redirection home
       | `Site ContSiteAdmin -> Redirection admin
       | `Site (ContSiteElection uuid) ->
@@ -96,7 +94,7 @@ module Make (Web_state : Web_state_sig.S) (Web_services : Web_services_sig.S) (P
                         let* () = Web_persist.clear_elections_by_owner_cache () in
                         return a
                      | Some x ->
-                        let account_last_connected = now () in
+                        let account_last_connected = Datetime.now () in
                         let x = {x with account_last_connected} in
                         let* () = Accounts.update_account x in
                         return x
@@ -112,20 +110,19 @@ module Make (Web_state : Web_state_sig.S) (Web_services : Web_services_sig.S) (P
        else
          restart_login ()
 
-  type pre_login_handler =
-    uuid option -> [`Username | `Address] -> auth_config -> state:string -> result Lwt.t
-
-  let pre_login_handlers = ref []
+  let auth_systems = ref ([] : (string * auth_system) list)
 
   let get_pre_login_handler uuid username_or_address kind a =
     let* state = generate_token () in
     let* () = Eliom_reference.set auth_env (Some (uuid, a, kind, state)) in
-    match List.assoc_opt a.auth_system !pre_login_handlers with
-    | Some handler -> handler uuid username_or_address a ~state
+    match List.assoc_opt a.auth_system !auth_systems with
+    | Some auth_system ->
+       let module X = (val auth_system uuid a) in
+       X.pre_login_handler username_or_address ~state
     | None -> fail_http `Not_found
 
-  let register_pre_login_handler ~auth_system handler =
-    pre_login_handlers := (auth_system, handler) :: !pre_login_handlers;
+  let register ~auth_system handler =
+    auth_systems := (auth_system, handler) :: !auth_systems;
     run_post_login_handler ~auth_system
 
   let rec find_auth_instance x = function
@@ -133,6 +130,30 @@ module Make (Web_state : Web_state_sig.S) (Web_services : Web_services_sig.S) (P
     | { auth_instance = i; _ } as y :: _ when i = x -> Some y
     | _ :: xs -> find_auth_instance x xs
 
+  let get_election_auth_configs uuid =
+    let* metadata = Web_persist.get_election_metadata uuid in
+    match metadata.e_auth_config with
+    | None -> return []
+    | Some x ->
+       x
+       |> List.map
+            (function
+             | {auth_system = "import"; auth_instance = name; _} ->
+                (match
+                   List.find_opt
+                     (function
+                      | `Export x -> x.auth_instance = name
+                      | _ -> false
+                     ) !Web_config.exported_auth_config
+                 with
+                 | Some (`Export x) -> [x]
+                 | _ -> []
+                )
+             | x -> [x]
+            )
+       |> List.flatten
+       |> return
+
   let login_handler service kind =
     let uuid = match kind with
       | `Site _ -> None
@@ -157,30 +178,7 @@ module Make (Web_state : Web_state_sig.S) (Web_services : Web_services_sig.S) (P
          match uuid with
          | None -> return (!Web_config.site_auth_config, `Site, `Username)
          | Some uuid ->
-            let* metadata = Web_persist.get_election_metadata uuid in
-            let* c =
-              match metadata.e_auth_config with
-              | None -> return []
-              | Some x ->
-                 x
-                 |> List.map
-                      (function
-                       | {auth_system = "import"; auth_instance = name; _} ->
-                          (match
-                             List.find_opt
-                               (function
-                                | `Export x -> x.auth_instance = name
-                                | _ -> false
-                               ) !Web_config.exported_auth_config
-                           with
-                           | Some (`Export x) -> [x]
-                           | _ -> []
-                          )
-                       | x -> [x]
-                      )
-                 |> List.flatten
-                 |> return
-            in
+            let* c = get_election_auth_configs uuid in
             let* username_or_address =
               let* voters = Web_persist.get_voters uuid in
               match voters with
@@ -245,4 +243,31 @@ module Make (Web_state : Web_state_sig.S) (Web_services : Web_services_sig.S) (P
     | None -> return @@ Html (Eliom_content.Html.F.div [])
     | Some a -> get_pre_login_handler None `Username (`Site ContSiteAdmin) a
 
+  let direct_voter_auth uuid x =
+    let fail () = failwith "invalid direct auth" in
+    let* c =
+      let* cs = get_election_auth_configs uuid in
+      match cs with
+      | [c] -> Lwt.return c
+      | _ ->
+         match x with
+         | `Assoc x ->
+            begin
+              match List.assoc_opt "service" x with
+              | Some (`String service) ->
+                 begin
+                   match find_auth_instance service cs with
+                   | Some c -> Lwt.return c
+                   | None -> fail ()
+                 end
+              | _ -> fail ()
+            end
+         | _ -> fail ()
+    in
+    match List.assoc_opt c.auth_system !auth_systems with
+    | Some auth_system ->
+       let module X = (val auth_system (Some uuid) c) in
+       let* user_name = X.direct x in
+       Lwt.return {user_name; user_domain = c.auth_instance}
+    | None -> fail ()
 end
