@@ -19,8 +19,7 @@
 (*  <http://www.gnu.org/licenses/>.                                       *)
 (**************************************************************************)
 
-open Belenios_core
-open Serializable_builtin_t
+open Belenios_core.Common
 open Web_serializable_t
 
 module type S = sig
@@ -32,7 +31,7 @@ module type S = sig
   val get_election_user : uuid -> user option Lwt.t
 
   val ballot : string option Eliom_reference.eref
-  val cast_confirmed : (string * Weight.t * bool, Web_common.error) result option Eliom_reference.eref
+  val cast_confirmed : (user * string * bool * Weight.t * bool, Web_common.error) result option Eliom_reference.eref
 
   val language : string option Eliom_reference.eref
 
