@@ -4,11 +4,10 @@ import unittest
 import random
 import os
 import sys
-from distutils.util import strtobool
 from selenium.webdriver.support.select import Select
 from util.fake_sent_emails_manager import FakeSentEmailsManager
 from util.selenium_tools import wait_for_element_exists, wait_for_an_element_with_partial_link_text_exists, wait_for_an_element_with_link_text_exists
-from util.election_testing import remove_database_folder, remove_election_from_database, wait_a_bit, build_css_selector_to_find_buttons_in_page_content_by_value, initialize_server, initialize_browser, verify_election_consistency, create_election_data_snapshot, delete_election_data_snapshot, log_in_as_administrator
+from util.election_testing import strtobool, remove_database_folder, remove_election_from_database, wait_a_bit, build_css_selector_to_find_buttons_in_page_content_by_value, initialize_server, initialize_browser, verify_election_consistency, create_election_data_snapshot, delete_election_data_snapshot, log_in_as_administrator
 from util.election_test_base import BeleniosElectionTestBase
 from util.execution import console_log, ConsoleLogDuration
 import settings
@@ -84,7 +83,11 @@ class BeleniosTestElectionScenario1(BeleniosElectionTestBase):
 
         wait_a_bit()
 
-        # FIXME: If no voter has cast their vote, it shows a "Internal Server Error" "Error 500" page
+        # She clicks on "Election home"
+        election_home_element = wait_for_an_element_with_link_text_exists(browser, "Election home", settings.EXPLICIT_WAIT_TIMEOUT)
+        election_home_element.click()
+
+        wait_a_bit()
 
         self.administrator_verifies_vote_results()
 
