@@ -7,12 +7,11 @@ import subprocess
 import re
 import sys
 from uuid import uuid4
-from distutils.util import strtobool
 from selenium.common.exceptions import UnexpectedAlertPresentException
 from selenium.webdriver.support.select import Select
 from util.fake_sent_emails_manager import FakeSentEmailsManager
-from util.selenium_tools import wait_for_element_exists, wait_for_element_exists_and_contains_expected_text, wait_for_element_exists_and_has_non_empty_content, wait_for_an_element_with_partial_link_text_exists, set_element_attribute, wait_for_element_exists_and_has_non_empty_attribute, verify_all_elements_have_attribute_value, verify_some_elements_have_attribute_value, wait_for_elements_exist_and_are_visible
-from util.election_testing import random_email_addresses_generator, remove_database_folder, remove_election_from_database, wait_a_bit, build_css_selector_to_find_buttons_in_page_content_by_value, initialize_server, initialize_browser, election_page_url_to_election_id, verify_election_consistency, create_election_data_snapshot, delete_election_data_snapshot, log_in_as_administrator, log_out, administrator_starts_creation_of_election, administrator_edits_election_questions, administrator_sets_election_voters, administrator_validates_creation_of_election
+from util.selenium_tools import wait_for_element_exists, wait_for_element_exists_and_contains_expected_text, wait_for_element_exists_and_has_non_empty_content, wait_for_an_element_with_partial_link_text_exists, set_element_attribute, wait_for_element_exists_and_has_non_empty_attribute, verify_all_elements_have_attribute_value, verify_some_elements_have_attribute_value, wait_for_elements_exist_and_are_visible, wait_for_an_element_with_link_text_exists
+from util.election_testing import strtobool, random_email_addresses_generator, remove_database_folder, remove_election_from_database, wait_a_bit, build_css_selector_to_find_buttons_in_page_content_by_value, initialize_server, initialize_browser, election_page_url_to_election_id, verify_election_consistency, create_election_data_snapshot, delete_election_data_snapshot, log_in_as_administrator, log_out, administrator_starts_creation_of_election, administrator_edits_election_questions, administrator_sets_election_voters, administrator_validates_creation_of_election
 from util.election_test_base import BeleniosElectionTestBase
 from util.execution import console_log, ConsoleLogDuration
 import settings
@@ -643,20 +642,16 @@ The election administrator.\
 
         wait_a_bit()
 
-        # She checks that the "DONE?" column of each trustee is to "Yes" (or if there is a threshold on trustees, she checks that at least `max_trustees` have a "Yes")
-        expected_label = "Yes"
-        yes_cells_selector = "#main table tr td:nth-last-child(1)"
-        attribute_name = "innerText"
         if max_trustees is None:
-            verify_all_elements_have_attribute_value(browser, yes_cells_selector, attribute_name, expected_label)
+            # She clicks on "Election home"
+            election_home_element = wait_for_an_element_with_link_text_exists(browser, "Election home", settings.EXPLICIT_WAIT_TIMEOUT)
+            election_home_element.click()
         else:
-            verify_some_elements_have_attribute_value(browser, yes_cells_selector, attribute_name, expected_label, max_trustees)
-
-        # She clicks on the "Compute the result" button
-        compute_result_button_expected_label = "Compute the result"
-        compute_result_button_css_selector = "#main input[type=submit][value='" + compute_result_button_expected_label + "']"
-        compute_result_button_element = wait_for_element_exists(browser, compute_result_button_css_selector)
-        compute_result_button_element.click()
+            # She clicks on the "Compute the result" button
+            compute_result_button_expected_label = "Compute the result"
+            compute_result_button_css_selector = "#main input[type=submit][value='" + compute_result_button_expected_label + "']"
+            compute_result_button_element = wait_for_element_exists(browser, compute_result_button_css_selector)
+            compute_result_button_element.click()
 
         wait_a_bit()
 
