@@ -16,6 +16,14 @@ from util.execution import console_log
 import settings
 
 
+def strtobool(s):
+    if s in ("y", "yes", "t", "true", "on", "1"):
+        return True
+    if s in ("n", "no", "f", "false", "off", "0"):
+        return False
+    raise ValueError("cannot interpret {} as a boolean".format(s))
+
+
 def random_email_addresses_generator(size=20):
     res = []
     for x in range(size):
@@ -24,7 +32,7 @@ def random_email_addresses_generator(size=20):
 
 
 def random_email_address_generator():
-    return random_generator() + "@mailinator.com"
+    return random_generator() + "@example.org"
 
 
 def random_generator(size=20, chars=string.ascii_lowercase + string.digits):
@@ -145,14 +153,14 @@ def repopulate_vote_confirmations_for_voters_from_sent_emails(fake_sent_emails_m
         Content-type: text/plain; charset="UTF-8"
         Content-transfer-encoding: quoted-printable
         From: Belenios public server <noreply@example.org>
-        To: "A6QKLFSL0TTJ05XE2LHD@mailinator.com"
-         <A6QKLFSL0TTJ05XE2LHD@mailinator.com>
+        To: "A6QKLFSL0TTJ05XE2LHD@example.org"
+         <A6QKLFSL0TTJ05XE2LHD@example.org>
         Subject: Your vote for election My test election for Scenario 1
         MIME-Version: 1.0
         X-Mailer: OcamlNet (ocamlnet.sourceforge.net)
         Date: Fri, 09 Nov 2018 21:40:39 +0100
 
-        Dear A6QKLFSL0TTJ05XE2LHD=40mailinator.com,
+        Dear A6QKLFSL0TTJ05XE2LHD=40example.org,
 
         Your vote for election
 
@@ -336,11 +344,10 @@ def belenios_tool_generate_credentials(election_id, number_of_voters=None, nh_qu
     generated_files_destination_folder = settings.GENERATED_FILES_DESTINATION_FOLDER
     belenios_tool_path = os.path.join(settings.GIT_REPOSITORY_ABSOLUTE_PATH, "_run/tool-debug/bin/belenios-tool")
     if nh_question:
-        group_base_path = "files/groups/rfc3526-2048.json"
+        crypto_group = "RFC-3526-2048"
     else:
-        group_base_path = "files/groups/default.json"
-    crypto_group_path = os.path.join(settings.GIT_REPOSITORY_ABSOLUTE_PATH, group_base_path)
-    command = [belenios_tool_path, "setup", "generate-credentials", "--uuid", election_id, "--group", crypto_group_path, "--count", str(number_of_voters)]
+        crypto_group = "BELENIOS-2048"
+    command = [belenios_tool_path, "setup", "generate-credentials", "--uuid", election_id, "--group", crypto_group, "--count", str(number_of_voters)]
     running_process = subprocess.Popen(command, cwd=generated_files_destination_folder, stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
     process_timeout = 15 * number_of_voters # seconds
     credential_file_id = None
@@ -420,8 +427,7 @@ def create_election_data_snapshot(election_id):
     else:
         raise Exception("Could not extract absolute path from output of mktemp:", out)
 
-    # Remark: If this command is run before any vote is cast, file `ballots.jsons` does not exist yet
-    subprocess.run(["cp", "election.json", "public_creds.txt", "trustees.json", "ballots.jsons", temporary_folder_absolute_path], cwd=election_folder) # TODO: Execute a command that works on other OS, like `shutil.copy()`
+    subprocess.run(["cp", election_id + ".bel", temporary_folder_absolute_path], cwd=election_folder) # TODO: Execute a command that works on other OS, like `shutil.copy()`
 
     return temporary_folder_absolute_path
 
