@@ -30,8 +30,8 @@ uuid="--uuid $UUID"
 group="--group RFC-3526-2048"
 
 # Generate credentials
-belenios-tool setup generate-credentials $uuid $group --count 102
-mv *.pubcreds public_creds.txt
+belenios-tool setup generate-credentials $uuid $group --count 102 | tee generate-credentials.out
+mv *.pubcreds public_creds.json
 mv *.privcreds private_creds.txt
 
 # Generate trustee keys
@@ -47,6 +47,19 @@ rm public_keys.jsons
 # Generate election parameters
 belenios-tool setup make-election $uuid $group --template $BELENIOS/tests/tool/templates/questions-mj.json
 
+# Initialize events
+belenios-tool archive init
+rm -f election.json trustees.json public_creds.json
+
+# Check public credential fingerprint
+EXPECTED_PUBLIC_CREDENTIAL_FINGERPRINT="$(tail -n1 generate-credentials.out| awk '{print $(NF)}')"
+ACTUAL_PUBLIC_CREDENTIAL_FINGERPRINT="$(tar -xOf $UUID.bel $(tar -tf $UUID.bel | head -n4 | tail -n1) | belenios-tool sha256-b64)"
+if [ "$EXPECTED_PUBLIC_CREDENTIAL_FINGERPRINT" != "$ACTUAL_PUBLIC_CREDENTIAL_FINGERPRINT" ]; then
+    echo "Discrepancy in public credential fingerprint"
+    exit 2
+fi
+rm -f generate-credentials.out
+
 header "Simulate votes"
 
 cat > votes.txt <<EOF
@@ -155,42 +168,40 @@ cat > votes.txt <<EOF
 EOF
 
 paste private_creds.txt votes.txt | while read id cred vote; do
-    belenios-tool election generate-ballot --privcred <(echo "$cred") --ballot <(echo "$vote")
+    belenios-tool election generate-ballot --privcred <(echo "$cred") --ballot <(echo "$vote") | belenios-tool archive add-event --type=Ballot
     echo "Voter $id voted" >&2
     echo >&2
-done > ballots.tmp
-mv ballots.tmp ballots.jsons
+done
 
 header "Perform verification"
 
 belenios-tool election verify
 
-header "Simulate and verify update"
+header "End voting phase"
 
-tdir="$(mktemp -d)"
-cp election.json public_creds.txt trustees.json "$tdir"
-head -n3 ballots.jsons > "$tdir/ballots.jsons"
-belenios-tool election verify-diff --dir1="$tdir" --dir2=.
-rm -rf "$tdir"
+belenios-tool archive add-event --type=EndBallots < /dev/null
+belenios-tool election compute-encrypted-tally | belenios-tool archive add-event --type=EncryptedTally
+belenios-tool election verify
 
 header "Shuffle ciphertexts"
 
-belenios-tool election shuffle > shuffles.jsons
+belenios-tool election shuffle --trustee-id=1 | belenios-tool archive add-event --type=Shuffle
 echo >&2
-belenios-tool election shuffle >> shuffles.jsons
+belenios-tool election shuffle --trustee-id=2 | belenios-tool archive add-event --type=Shuffle
+belenios-tool archive add-event --type=EndShuffles < /dev/null
 
 header "Perform decryption"
 
+trustee_id=1
 for u in *.privkey; do
-    belenios-tool election decrypt --privkey $u
+    belenios-tool election decrypt --privkey $u --trustee-id $trustee_id | belenios-tool archive add-event --type=PartialDecryption
     echo >&2
-done > partial_decryptions.tmp
-mv partial_decryptions.tmp partial_decryptions.jsons
+    : $((trustee_id++))
+done
 
 header "Finalize tally"
 
-belenios-tool election compute-result
-rm -f shuffles.jsons
+belenios-tool election compute-result | belenios-tool archive add-event --type=Result
 
 header "Perform final verification"
 
@@ -202,8 +213,9 @@ cat > mj.reference <<EOF
 {"raw":[[30,25,20,15,10],[25,30,20,14,11],[6,24,40,24,6],[10,15,20,25,30]],"valid":100,"blank":1,"invalid":[[5,0,5,5]],"winners":[[0],[1],[2],[3]]}
 EOF
 
+RESULT=$(tar -tf $UUID.bel | tail -n2 | head -n1)
 if command -v jq > /dev/null; then
-    if diff -u mj.reference <(jq --compact-output '.result[0]' < result.json | belenios-tool method majority-judgment --ngrades 5 --blank-allowed true); then
+    if diff -u mj.reference <(tar -xOf $UUID.bel $RESULT | jq --compact-output '.result[0]' | belenios-tool method majority-judgment --ngrades 5 --blank-allowed true); then
         echo "Majority Judgment output is identical!"
     else
         echo "Differences in Majority Judgment output!"
@@ -217,7 +229,7 @@ echo
 echo "=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-="
 echo
 echo "The simulated election was successful! Its result can be seen in"
-echo "  $DIR/result.json"
+echo "  $DIR"
 echo
 echo "=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-="
 echo
